<?php
// $Revision: 1.1.2.1 $
//  ------------------------------------------------------------------------ //
//  XooNIps - Neuroinformatics Base Platform System                          //
//  Copyright (C) 2005-2007 RIKEN, Japan All rights reserved.                //
//  http://xoonips.sourceforge.jp/                                           //
//  ------------------------------------------------------------------------ //
//  This program is free software; you can redistribute it and/or modify     //
//  it under the terms of the GNU General Public License as published by     //
//  the Free Software Foundation; either version 2 of the License, or        //
//  (at your option) any later version.                                      //
//                                                                           //
//  You may not change or alter any portion of this comment or credits       //
//  of supporting developers from this source code or any supporting         //
//  source code which is considered copyrighted (c) material of the          //
//  original comment or credit authors.                                      //
//                                                                           //
//  This program is distributed in the hope that it will be useful,          //
//  but WITHOUT ANY WARRANTY; without even the implied warranty of           //
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            //
//  GNU General Public License for more details.                             //
//                                                                           //
//  You should have received a copy of the GNU General Public License        //
//  along with this program; if not, write to the Free Software              //
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA //
//  ------------------------------------------------------------------------ //

if ( ! defined( 'XOOPS_ROOT_PATH' ) ) exit();

// check token ticket
require_once( '../class/base/gtickets.php' );
$ticket_area = 'xoonips_admin_maintenance_item_delete';
if ( ! $xoopsGTicket->check( true, $ticket_area, false ) ) {
  redirect_header( $xoonips_admin['mypage_url'], 3, $xoopsGTicket->getErrors() );
  exit();
}

// load libraries
include '../include/lib.php';
include '../include/libitem.php';

// get requests
$get_keys = array(
  'tree' => array( 'i', true, false ),
);
$get_vals = xoonips_admin_get_requests( 'post', $get_keys );
$tree_ids = $get_vals['tree'];

// function
function xoonips_admin_maintenance_item_delete_item( $iid ) {
  $item_basic = xnpitmgrGetItemBasicInfo($iid);
  if ( $item_basic === false ) {
    // unexpected error occured in sql query
    return false;
  }
  if ( count( $item_basic) == 0 ) {
    // basic information not found : it means item not registered
    return false; 
  }
  $item_type = xnpitmgrGetItemTypeNameById( $item_basic['item_type_id'] );
  if ( $item_type === false ) {
    // unexpected error occered in sql query
    return false;
  }
  if ( $item_type['viewphp'] ==  null ) {
    // viewphp file not defined : maybe this is index
    return false;
  }
  $viewphp_file = XOOPS_ROOT_PATH.'/modules/'.$item_type['viewphp'];
  if ( ! file_exists( $viewphp_file ) ) {
    // viewphp file not found
    return false;
  }
  include_once $viewphp_file;
  $delete_func = $item_type['name'].'DeleteItem';
  if ( ! function_exists( $delete_func ) ) {
    // delete function not defined
    return false;
  }
  return $delete_func( $iid );
}

// logic
$empty_tree_ids = true;
$results = array();
$xnpsid = $_SESSION['XNPSID'];
if ( count( $tree_ids ) > 0 ) {
  $myts =& MyTextSanitizer::getInstance();
  $treelist = xnpitmgrListIndexTree( XNPITMGR_LISTMODE_PRIVATEONLY );
  $treemap = array();
  foreach ( $treelist as $tree ) {
    $treemap[$tree['id']] = $tree['fullpath'];
  }
  $empty_tree_ids = false;
  $evenodd = 'odd';
  // execute item delete
  foreach ( $tree_ids as $xid ) {
    $succeed = 0;
    $failed = 0;
    $iids = xnpitmgrListIndexItems( array( $xid ) );
    if ( $iids === false ) continue; // no item in tree
    foreach ( $iids as $iid ) {
      $res = xoonips_admin_maintenance_item_delete_item( $iid );
      if ( $res === true ) {
        // succeed
        $succeed++;
      } else {
        // error occured
        $failed++;
      }
    }
    $results[] = array(
      'id' => $xid, 
      'evenodd' => $evenodd,
      'index' => $myts->htmlSpecialChars( $treemap[$xid] ),
      'succeed' => $succeed,
      'failed' => $failed,
    );
    $evenodd = ( $evenodd == 'even' ) ? 'odd' : 'even';
  }
}

// title
$title = _AM_XOONIPS_MAINTENANCE_ITEM_DUPDATE_TITLE;

// breadcrumbs
$breadcrumbs = array(
  array( 'type' => 'top',  'label' => _AM_XOONIPS_TITLE, 'url' => $xoonips_admin['admin_url'].'/' ),
  array( 'type' => 'link',  'label' => _AM_XOONIPS_MAINTENANCE_TITLE, 'url' => $xoonips_admin['myfile_url'] ),
  array( 'type' => 'link',  'label' => _AM_XOONIPS_MAINTENANCE_ITEM_TITLE, 'url' => $xoonips_admin['mypage_url'] ),
  array( 'type' => 'link',  'label' => _AM_XOONIPS_MAINTENANCE_ITEM_DELETE_TITLE, 'url' => $xoonips_admin['mypage_url'].'&amp;action=delete' ),
  array( 'type' => 'label', 'label' => $title, 'url' => '' )
);

// templates
require_once( '../class/base/pattemplate.class.php' );
$tmpl = new PatTemplate();
$tmpl->setBaseDir( 'templates' );
$tmpl->readTemplatesFromFile( 'maintenance_item_dupdate.tmpl.html' );
$tmpl->addVar( 'header', "TITLE", $title );
$tmpl->setAttribute( 'breadcrumbs', 'visibility', 'visible' );
$tmpl->addRows( 'breadcrumbs_items', $breadcrumbs );
$tmpl->addVar( 'main', 'index', _AM_XOONIPS_MAINTENANCE_ITEM_LABEL_INDEX );
$tmpl->addVar( 'main', 'succeed', _AM_XOONIPS_MAINTENANCE_ITEM_LABEL_SUCCEED );
$tmpl->addVar( 'main', 'failed', _AM_XOONIPS_MAINTENANCE_ITEM_LABEL_FAILED );
if ( $empty_tree_ids ) {
  $tmpl->setAttribute( 'empty_results', 'visibility', 'visible' );
  $tmpl->setAttribute( 'results', 'visibility', 'hidden' );
  $tmpl->addVar( 'empty_results', 'empty', _AM_XOONIPS_MAINTENANCE_ITEM_INDEX_EMPTY );
} else {
  $tmpl->addRows( 'results', $results );
}
$tmpl->addVar( 'main', 'back', _AM_XOONIPS_LABEL_BACK );

xoops_cp_header();
$tmpl->displayParsedTemplate( 'main' );
xoops_cp_footer();

?>
