<?php
// $Revision: 1.1.2.12 $
//  ------------------------------------------------------------------------ //
//  XooNIps - Neuroinformatics Base Platform System                          //
//  Copyright (C) 2005-2007 RIKEN, Japan All rights reserved.                //
//  http://xoonips.sourceforge.jp/                                           //
//  ------------------------------------------------------------------------ //
//  This program is free software; you can redistribute it and/or modify     //
//  it under the terms of the GNU General Public License as published by     //
//  the Free Software Foundation; either version 2 of the License, or        //
//  (at your option) any later version.                                      //
//                                                                           //
//  You may not change or alter any portion of this comment or credits       //
//  of supporting developers from this source code or any supporting         //
//  source code which is considered copyrighted (c) material of the          //
//  original comment or credit authors.                                      //
//                                                                           //
//  This program is distributed in the hope that it will be useful,          //
//  but WITHOUT ANY WARRANTY; without even the implied warranty of           //
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            //
//  GNU General Public License for more details.                             //
//                                                                           //
//  You should have received a copy of the GNU General Public License        //
//  along with this program; if not, write to the Free Software              //
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA //
//  ------------------------------------------------------------------------ //

include_once XOOPS_ROOT_PATH . '/class/xml/rpc/xmlrpcapi.php';
include_once XOOPS_ROOT_PATH . '/modules/xoonips/include/functions.php';
include_once XOOPS_ROOT_PATH . '/modules/xoonips/condefs.php';
// for RSS update
include_once XOOPS_ROOT_PATH . '/modules/xoonips/include/lib.php';
include_once XOOPS_ROOT_PATH . '/modules/xoonips/include/AL.php';

/**
 *
 * base class of Business Logic
 *
 */
class XooNIpsLogic
{
    function XooNIpsLogic() 
    {
    }

    /**
     * implement of logic
     * @abstract
     * @param[in] array $vars input parameters of logic
     * @param[out] XooNIpsResponse $response result of logic(success/fault, response, error)
     */
    function execute(&$vars, &$response) 
    {
        /* abstract */
        return false;
    }

    /**
     * @brief validate $sessionid and restore $_SESSION.
     *
     * @param[in]  $sessionid session ID
     * @return     array($result,$uid,$session)
     *             if $result==true && uid != UID_GUEST, $sessionid was valid, $uid and $session are user ID and XooNIpsSession object.
     *             if $result==true && uid == UID_GUEST, $sessionid was valid and $session=false.
     *             if $result==false,                    $sessionid was invalid and $uid=$session=false.
     *
     */
    function restoreSession($sessionid) 
    {
        global $_SESSION;
        session_id($sessionid);
        // restore $_SESSION
        $xoops_sess_handler = &xoops_gethandler('session');
        $sess_data = $xoops_sess_handler->read($sessionid);
        if ($sess_data == '') {
            return array(
                false,
                false,
                false
            );
        } else {
            session_decode($sess_data);
        }
        $uid = $_SESSION['xoopsUserId'];
        if ($uid == UID_GUEST) {
            // reject guest if guest is forbidden
            $config_handler = &xoonips_getormhandler('xoonips', 'config');
            $configs = &$config_handler->getObjects(new Criteria('name', XNP_CONFIG_PUBLIC_ITEM_TARGET_USER_KEY));
            if (is_array($configs) && count($configs) == 1 && $configs[0]->get('value') != XNP_CONFIG_PUBLIC_ITEM_TARGET_USER_ALL) {
                return array(
                    false,
                    false,
                    false
                );
            }
            $session = false;
        } else {
            // validate session
            $sess_handler = &xoonips_getormhandler('xoonips', 'session');
            $sessions = &$sess_handler->getObjects(new Criteria('sess_id', $sessionid));
            if (!$sessions || count($sessions) != 1) {
                return array(
                    false,
                    false,
                    false
                );
            }
            $session = $sessions[0];
            $session->setVar('updated', time());
            $sess_handler->insert( $session );
        }
        return array(
            true,
            $uid,
            $session
        );
    }

    /** 
     * @brief helper function for removeFile, updateFile, updateItem, putItem. update certify_state, notify, insert event, update item_status, update rss
     *
     * @param response XooNIpsResponse object
     * @param item XooNIpsItem object
     * @return result true if succeeded.
     */
    function touchItem(&$error, &$item, $uid) 
    {
        /*
        foreach ( public or group index ){
        certify_state = CERTIFY_REQUIRED
        insert event ETID_REQUEST_CERTIFY_ITEM
        if ( auto_certify ){
        certify_state = CERTIFIED
        insert event ETID_CERTIFY_ITEM
        }
        else {
        send notification to moderator/group_admin
        }
        }
        
        if ( public -> nonpublic ){
        update item_status table set ...
        update RSS
        }
        if ( nonpublic -> public ){
        update item_status table set ...
        update RSS
        }
        */
        // get indexes
        $index_ids = array();
        $index_item_links = $item->getVar('indexes');
        if (!$index_item_links || count($index_item_links) == 0) {
            $error->add(XNPERR_SERVER_ERROR, "no indexes");
            return false;
        }
        // certify automatically?
        $config_handler = &xoonips_getormhandler('xoonips', 'config');
        $certify_item = $config_handler->get('certify_item'); // todo define string const
        if (!$certify_item) {
            $error->add(XNPERR_SERVER_ERROR, "no certify_item config");
            return false;
        }
        $auto_certify = ($certify_item->get('value') == 'auto');
        // get moderator uid list
        $user_handler = &xoonips_gethandler('user');
        $config = &$config_handler->get('moderator_gid'); // todo: define string constant
        if ($config == false) {
            $error->add(XNPERR_SERVER_ERROR, "no moderator group setting");
            return false;
        }
        $moderator_gid = $config->get('value');
        $xoops_member_handler = &xoops_gethandler('member');
        $moderator_uid_list = $xoops_member_handler->getUsersByGroup($moderator_gid);
        $index_handler = &xoonips_getormhandler('xoonips', 'index');
        $event_log_handler = &xoonips_getormhandler('xoonips', 'event_log');
        $index_item_link_handler = &xoonips_getormhandler('xoonips', 'index_item_link');
        $is_public = false;
        $len = count($index_item_links);
        $basic = $item->getVar('basic');
        $item_id = $basic->get('item_id');
        for ($i = 0; $i < $len; $i++) {
            $index_item_link = &$index_item_links[$i];
            $certify_state = $index_item_link->get('certify_state');
            $index_id = $index_item_link->get('index_id');
            $index = $index_handler->get($index_id);
            if (!$index) {
                // maybe someone deleted this index.
                $error->add(XNPERR_SERVER_ERROR, "item is in non-existent index(index_id=$index_id)");
                return false;
            }
            $open_level = $index->get('open_level');
            if ($open_level == OL_GROUP_ONLY || $open_level == OL_PUBLIC) {
                // get moderator/groupadmin uid list
                $open_level = $index->get('open_level');
                $uid_list = array();
                if ($open_level == OL_PUBLIC) {
                    $uid_list = $moderator_uid_list;
                    if ($auto_certify) {
                        $is_public = true;
                    }
                } else if ($open_level == OL_GROUP_ONLY) {
                    $gid = $index->get('gid');
                    $users_handler = &xoonips_getormhandler('xoonips', 'users');
                    $criteria = new CriteriaCompo();
                    $criteria->add(new Criteria('gid', $gid));
                    $criteria->add(new Criteria('is_admin', 1));
                    $join = new XoopsJoinCriteria($GLOBALS['xoopsDB']->prefix('xoonips_groups_users_link') , 'uid', 'uid');
                    $users = $users_handler->getObjects($criteria, false, '', false, $join);
                    if (!$users) {
                        $error->add(XNPERR_SERVER_ERROR, "cannot get group admins(gid=$gid)");
                        return false;
                    }
                    foreach($users as $user) {
                        $uid_list[] = $user->get('uid');
                    }
                }
                $indexes[] = $index;
                $event_log = $event_log_handler->create();
                $event_log->setVar('event_type_id', ETID_REQUEST_CERTIFY_ITEM);
                $event_log->setVar('timestamp', time());
                $event_log->setVar('exec_uid', $uid);
                $event_log->setVar('remote_host', xnpGetRemoteHost());
                $event_log->setVar('index_id', $index_id);
                $event_log->setVar('item_id', $item_id);
                if (!$event_log_handler->insert($event_log)) {
                    $error->add(XNPERR_SERVER_ERROR, "cannot insert event");
                    return false;
                }
                if ($auto_certify) {
                    $index_item_link->setVar('certify_state', CERTIFIED);
                    $event_log = $event_log_handler->create();
                    $event_log->setVar('event_type_id', ETID_CERTIFY_ITEM);
                    $event_log->setVar('timestamp', time());
                    $event_log->setVar('exec_uid', $uid);
                    $event_log->setVar('remote_host', xnpGetRemoteHost());
                    $event_log->setVar('index_id', $index_id);
                    $event_log->setVar('item_id', $item_id);
                    if (!$event_log_handler->insert($event_log)) {
                        $error->add(XNPERR_SERVER_ERROR, "cannot insert event");
                        return false;
                    }
                } else {
                    $index_item_link->setVar('certify_state', CERTIFY_REQUIRED);
                    // send notiication message
                    // copy from lib.php xnpUpdateIndex()
                    global $xoopsModule;
                    $notification_handler = &xoops_gethandler('notification');
                    //define tags here for notification message
                    $tags = array();
                    $tags['ITEM_URL'] = XOOPS_URL . "/modules/xoonips/detail.php?item_id=${item_id}";
                    $tags['CERTIFY_URL'] = XOOPS_URL . "/modules/xoonips/certify.php";
                    $tags['INDEX'] = xnpGetIndexPathString(session_id() , $index_id);
                    $notification_handler = &xoops_gethandler('notification');
                    $result = $notification_handler->triggerEvent('administrator', 0, 'item_certify_request', $tags, $uid_list);
                }
                if (!$index_item_link_handler->insert($index_item_link)) {
                    $error->add(XNPERR_SERVER_ERROR, "cannot update index_item_link");
                    return false;
                }
            }
        }
        
        // update item_status, update RSS
        $item_status_handler = &xoonips_getormhandler('xoonips', 'item_status');
        $old_item_status = $item_status_handler->get($item_id);
        $item_status = $item_status_handler->create(); // re-create because XooNIpsTableObject cannot be setVar(key,null)ed
        $item_status->setVar('item_id', $item_id);
        if ( $old_item_status ){
            $item_status->unsetNew();
        }
        if ( $is_public ){
            // public -> public or non-public -> public
            $item_status->setVar('created_timestamp', time());
            $item_status->setVar('is_deleted', 0);
            if (!$item_status_handler->insert($item_status)) {
                $error->add(XNPERR_SERVER_ERROR, "cannot update item_status");
                return false;
            }
            xnpGenerateRss(); //update RSS
        }
        else {
            if ($old_item_status && $old_item_status->get('is_deleted') == 0) {
                // public -> non-public
                $item_status->setVar('deleted_timestamp', time());
                $item_status->setVar('is_deleted', 1);
                if (!$item_status_handler->insert($item_status)) {
                    $error->add(XNPERR_SERVER_ERROR, "cannot update item_status");
                    return false;
                }
                xnpGenerateRss(); //update RSS
            }
            else {
                // non-public -> non-public
            }
        }
        
        return true;
    }
    /* todo: test
    *  result contains:
    *   my private
    *   my private, group(not certified)
    *   my private, public(not certified)
    *  result does not contain:
    *   other private
    *   my private, group(certified)
    *   my private, public(certified)
    */

    /** get ids of private(not certified in any group/public index) item. 
     * @param uid
     * @return item_id[]
     */
    function getPrivateItemIds($uid) 
    {
        $index_item_link_handler = &xoonips_getormhandler('xoonips', 'index_item_link');
        $join = new XoopsJoinCriteria($GLOBALS['xoopsDB']->prefix('xoonips_item_basic') , 'item_id', 'item_id', 'left', 'tb');
        $criteria = new Criteria('uid', $uid);
        $criteria->setGroupby('item_id');
        $index_item_links = $index_item_link_handler->getObjects($criteria, false, 'tb.item_id as item_id, sum(certify_state=' . CERTIFIED . ') as certified_count', false, $join, 'certified_count=0');
        $iids = array();
        foreach($index_item_links as $link) {
            $iids[] = $link->get('item_id');
        }
        return $iids;
    }

    /** get ids of group-certified item. 
     * @param gid
     * @return item_id[]
     */
    function getCertifiedGroupItemIds($gid) 
    {
        $index_item_link_handler = &xoonips_getormhandler('xoonips', 'index_item_link');
        $join = new XoopsJoinCriteria($GLOBALS['xoopsDB']->prefix('xoonips_index') , 'index_id', 'index_id');
        $criteria = new CriteriaCompo();
        $criteria->add(new Criteria('gid', $gid));
        $criteria->add(new Criteria('open_level', OL_GROUP_ONLY));
        $criteria->add(new Criteria('certify_state', CERTIFIED));
        $criteria->setGroupby('item_id');
        $index_item_links = $index_item_link_handler->getObjects($criteria, false, 'item_id', false, $join);
        $iids = array();
        foreach($index_item_links as $link) {
            $iids[] = $link->get('item_id');
        }
        return $iids;
    }

    /** get total file size of items. 
     * @param item_ids item_id[]
     * @return size(bytes)
     */
    function getTotalSizeOfItems($item_ids) 
    {
        $total_size = 0;
        if (count($item_ids)) {
            $criteria = new CriteriaCompo();
            $file_handler = &xoonips_getormhandler('xoonips', 'file');
            $criteria->add(new Criteria('item_id', '(' . implode(',', $item_ids) . ')', 'in'));
            $criteria->add(new Criteria('is_deleted', 0));
            $files = $file_handler->getObjects($criteria);
            foreach($files as $file) {
                $total_size+= $file->get('file_size');
            }
        }
        return $total_size;
    }

    /**
     * @brief get size of item
     * @param item[in] XooNIpsItemCompo
     */
    function getSizeOfItem($item) 
    {
        $basic = $item->getVar('basic');
        $item_type_handler = &xoonips_getormhandler('xoonips', 'item_type');
        $item_type = $item_type_handler->get($basic->get('item_type_id'));
        if (!$item_type) {
            return false;
        }
        $detail_item_type_handler = &xoonips_getormhandler($item_type->get('name') , 'item_type');
        if (!$detail_item_type_handler) {
            return false;
        }
        $detail_item_type = $detail_item_type_handler->get($basic->get('item_type_id'));
        if (!$detail_item_type) {
            return false;
        }
        $result = 0;
        foreach($detail_item_type->getFileTypeNames() as $file_type_name) {
            $files = $item->getVar($file_type_name);
            if (is_array($files)) {
                foreach($files as $file) {
                    $result+= $file->get('file_size');
                }
            } else if ($files) {
                $result+= $files->get('file_size');
            }
        }
        return $result;
    }
    /* todo: test
    *  if ( sizeof(private_iids) + new_size - old_size > private_item_storage_limit ) -> error: private item storage limit exceeds
    *  if ( !$old_index_item_links && count(private_iids) + 1 > private_item_number_limit ) -> error: private item number limit exceeds
    *  if (  old && new && private                 ) -> check private storage(+new-old)
    *  if ( !old && new && private                 ) -> check private storage(+new    ), check private num(+1)
    *  if (  old && new && group   && auto_certify ) -> check group storage(+new-old)
    *  if ( !old && new && group   && auto_certify ) -> check group storage(+new    ), check group num(+1)
    *  new: no permission -> -
    *  old: no permission -> -
    *  new: no index -> error: cannot get index
    *  old: no index -> error: cannot get index
    *  user: no user -> error: no such user
    */

    /**
     * @brief check if enough space is available
     * @param error[out] error information
     * @param uid[in] user id
     * @param new_size[in]  future total file size of item
     * @param new_index_item_links[in]  where item will be registered to
     * @param old_size[in]  current total file size of item
     * @param old_index_item_links[in]  where item is registered to
     */
    function isEnoughSpace(&$error, $uid, $new_size, $new_index_item_links, $old_size = 0, $old_index_item_links = array()) 
    {
        $result = true;
        // check private index limit
        $user_handler = &xoonips_getormhandler('xoonips', 'users');
        $user = $user_handler->get($uid);
        if (!$user) {
            $error->add(XNPERR_SERVER_ERROR, "no such user(uid=$uid)");
            return false;
        }
        $private_iids = $this->getPrivateItemIds($uid);
        if ($this->getTotalSizeOfItems($private_iids) +$new_size-$old_size > $user->get('private_item_storage_limit')) {
            $error->add(XNPERR_SERVER_ERROR, "private item storage limit exceeds(uid=$uid)");
            $result = false;
        }
        if (!$old_index_item_links) {
            if (count($private_iids) +1 > $user->get('private_item_number_limit')) {
                $error->add(XNPERR_SERVER_ERROR, "private item number limit exceeds(uid=$uid)");
                $result = false;
            }
        }
        // get group ids
        $index_handler = &xoonips_getormhandler('xoonips', 'index');
        $new_gids = array();
        foreach($new_index_item_links as $link) {
            $index_id = $link->get('index_id');
            $index = $index_handler->get($index_id);
            if (!$index) {
                $error->add(XNPERR_SERVER_ERROR, "cannot get index(index_id=$index_id)");
                return false;
            } else if ($index->get('open_level') == OL_GROUP_ONLY) {
                $new_gids[$index->get('gid') ] = true;
            }
        }
        $old_gids = array();
        foreach($old_index_item_links as $link) {
            $index_id = $link->get('index_id');
            $index = $index_handler->get($index_id);
            if (!$index) {
                $error->add(XNPERR_SERVER_ERROR, "cannot get index(index_id=$index_id)");
                return false;
            } else if ($index->get('open_level') == OL_GROUP_ONLY) {
                $old_gids[$index->get('gid') ] = true;
            }
        }
        // item number/storage limit check (group) (only if auto_certify)
        $config_handler = &xoonips_getormhandler('xoonips', 'config');
        $config = $config_handler->get('certify_item');
        if (!$config) {
            $error->add(XNPERR_SERVER_ERROR, "cannot get certify_item config");
            return false;
        } else if ($config->get('value') == 'auto' && count($new_gids)) {
            $groups_handler = &xoonips_getormhandler('xoonips', 'groups');
            foreach($new_gids as $gid => $dummy) {
                $group = $groups_handler->get($gid);
                $group_iids = $this->getCertifiedGroupItemIds($gid);
                if ($this->getTotalSizeOfItems($group_iids) +$new_size-(empty($old_gids[$gid]) ? 0 : $old_size) > $group->get('group_item_storage_limit')) {
                    $error->add(XNPERR_SERVER_ERROR, "group item storage limit exceeds(gid=$gid)");
                    $result = false;
                }
                if (empty($old_gids[$gid])) {
                    if (count($group_iids) +1 > $group->get('group_item_number_limit')) {
                        $error->add(XNPERR_SERVER_ERROR, "group item number limit exceeds(gid=$gid)");
                        $result = false;
                    }
                }
            }
        }
        return $result;
    }
    
    /**
     * @brief guess mime-type of file
     * @param file XooNIpsFile
     * @return mime-type string(e.g. application/pdf)
     */
    function guessMimeType($file) 
    {
        if (!extension_loaded('fileinfo')) {
            if (!@dl('fileinfo.so')) {
                return false;
            }
        }
        $config_handler = &xoonips_getormhandler('xoonips', 'config');
        $config = $config_handler->get('magic_file_path');
        if ( !$config ){
            return false;
        }
        $finfo = finfo_open(FILEINFO_MIME, $config->get('value'));
        if ( !$finfo ){
            return false;
        }
        $result = finfo_file($finfo, $file->getFilepath());
        finfo_close($finfo);
        return $result;
    }

    /**
     * @brief create thumbnail image of file
     *
     */
    function createThumbnail(&$error, &$file) 
    {
        // load gd
        if (!extension_loaded('gd')) {
            if (!@dl('gd.so')) {
                $error->add(XNPERR_SERVER_ERROR, "cannot load gd");
                return false;
            }
        }
        $fileName = $file->getFilepath();
        $type = $file->get('mime_type');
        // The file type is judged from extensions.
        $ar = pathinfo($file->get('original_file_name'));
        $ext = strtolower($ar['extension']);
        if (function_exists('imagecreatefromjpeg') && ($type == 'image/jpeg' || $ext == 'jpg' || $ext == 'jpeg' || $ext == 'jpe')) {
            $imageID = @imagecreatefromjpeg($fileName);
        } else if (function_exists('imagecreatefromgif') && ($type == 'image/gif' || $ext == 'gif')) {
            $imageID = @imagecreatefromgif($fileName);
        } else if (function_exists('imagecreatefrompng') && ($type == 'image/png' || $ext == 'png')) {
            $imageID = @imagecreatefrompng($fileName);
        } else {
            // unknown image formats
            $error->add(XNPERR_INVALID_PARAM, "unknown image file format");
            return false;
        }
        if (!$imageID) {
            // failure in loading images.
            $error->add(XNPERR_INVALID_PARAM, "bad image file");
            return false;
        }
        $width = imagesx($imageID);
        $height = imagesy($imageID);
        $maxWidth = 100; // maximum file size in thumbnail
        $maxHeight = 100;
        if ($maxWidth < $width || $maxHeight < $height) { // If size of image file is too large, need to reduce it.
            $scaleX = $maxWidth/$width;
            $scaleY = $maxHeight/$height;
            $scale = min($scaleX, $scaleY);
            $newWidth = round($width*$scale);
            $newHeight = round($height*$scale);
            // resize
            $newImageID = imagecreatetruecolor($newWidth, $newHeight);
            $result = imagecopyresized($newImageID, $imageID, 0, 0, 0, 0, $newWidth, $newHeight, $width, $height);
            $imageID = $newImageID;
            $width = $newWidth;
            $height = $newHeight;
        }
        $xnpsid = session_id();
        $tmpfile = "/tmp/XooNIpsThumbnail_${xnpsid}_" . md5(uniqid(rand() , 1)); // make a unique filename.
        if (false == imagepng($imageID, $tmpfile)) {
            $error->add(XNPERR_SERVER_ERROR, "cannot output image file");
            return false;
        }
        $result = file_get_contents($tmpfile);
        unlink($tmpfile);
        $file->set('thumbnail_file', $result);
        return true;
    }
    
    /** convert prefixed number to a number(e.g. '4k' to 4096)
     */
    function returnBytes($val) {
        if ( $val == '' || $val == -1 ) {
            // '' : disable memory limit.
            // -1 : unlimit size
            $val = '1G';
        }
        $val = trim($val);
        $len = strlen( $val );
        $last = strtolower($val{$len-1});
        if ( ! is_numeric( $last ) ) {
            $val = substr( $val, 0, $len-1 );
            switch( $last ) {
                // The 'G' modifier is available since PHP 5.1.0
            case 'g':
                $val *= 1024;
            case 'm':
                $val *= 1024;
            case 'k':
                $val *= 1024;
            }
        }
        return $val;
    }
}
?>
