<?php
// $Revision: 1.1.2.3 $
//  ------------------------------------------------------------------------ //
//  XooNIps - Neuroinformatics Base Platform System                          //
//  Copyright (C) 2005-2007 RIKEN, Japan All rights reserved.                //
//  http://xoonips.sourceforge.jp/                                           //
//  ------------------------------------------------------------------------ //
//  This program is free software; you can redistribute it and/or modify     //
//  it under the terms of the GNU General Public License as published by     //
//  the Free Software Foundation; either version 2 of the License, or        //
//  (at your option) any later version.                                      //
//                                                                           //
//  You may not change or alter any portion of this comment or credits       //
//  of supporting developers from this source code or any supporting         //
//  source code which is considered copyrighted (c) material of the          //
//  original comment or credit authors.                                      //
//                                                                           //
//  This program is distributed in the hope that it will be useful,          //
//  but WITHOUT ANY WARRANTY; without even the implied warranty of           //
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            //
//  GNU General Public License for more details.                             //
//                                                                           //
//  You should have received a copy of the GNU General Public License        //
//  along with this program; if not, write to the Free Software              //
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA //
//  ------------------------------------------------------------------------ //

if (!defined('XOOPS_ROOT_PATH')) exit();

/**
 * The basic object class for the XooNIps
 *
 */
class XooNIpsTableObject extends XoopsObject
{

    /**
     * extra vars holder for the joining tables
     * @var array
     * @access private
     */
    var $_extra_vars = array();

    /**
     * constructor
     *
     * normally, this is called from child classes only
     * @access public
     */
    function XooNIpsTableObject() 
    {
        $this->XoopsObject();
    }

    /**
     * set text area display attributes
     *
     * @access public
     * @parem bool $dohtml use raw html
     * @parem bool $doxcode use xcode
     * @parem bool $dosmiley use smiley marks
     * @parem bool $dobr use <br /> new line
     */
    function setTextAreaDisplayAttributes($dohtml, $doxcode, $dosmiley, $dobr) 
    {
        $this->vars['dohtml']['value'] = ($dohtml === true) ? 1 : 0;
        $this->vars['dohtml']['changed'] = false;
        $this->vars['doxcode']['value'] = ($doxcode === true) ? 1 : 0;
        $this->vars['doxcode']['changed'] = false;
        $this->vars['dosmiley']['value'] = ($dosmiley === true) ? 1 : 0;
        $this->vars['dosmiley']['changed'] = false;
        $this->vars['dobr']['value'] = ($dobr === true) ? 1 : 0;
        $this->vars['dobr']['changed'] = false;
    }

    /**
     * set a value to a variable
     *
     * @access public
     * @param string $key name of the variable to assign
     * @param mixed $value value to assign
     * @param bool $not_gpc
     */
    function setVar($key, $value, $not_gpc = false) 
    {
        if (!empty($key) && isset($value) && isset($this->vars[$key])) {
            $this->vars[$key]['value'] = $not_gpc ? $value : $this->_stripSlashesGPC($value);
            $this->vars[$key]['not_gpc'] = true; // always true
            $this->vars[$key]['changed'] = true;
            $this->setDirty();
        }
    }

    /**
     * get all variables as array
     *
     * @access public
     * @param string $format format to use for the output
     * @return array reference to the variables array
     */
    function getVarArray($format) 
    {
        $vars_array = array();
        $keys_array = &$this->getKeysArray();
        foreach($keys_array as $k) {
            $vars_array[$k] = $this->getVar($k, $format);
        }
        return $vars_array;
    }

    /**
     * get all keys as array
     *
     * @access public
     * @return array keys array
     */
    function &getKeysArray() 
    {
        $ignore_keys = array(
            'dohtml',
            'doxcode',
            'dosmiley',
            'dobr'
        );
        $keys_array = array_diff(array_keys($this->vars) , $ignore_keys);
        return $keys_array;
    }

    /**
     * strip slashes when magic quota gpc is on
     *
     * @access private
     * @param mixed $value
     * @return mixed slashes striped value
     */
    function _stripSlashesGPC($value) 
    {
        return get_magic_quotes_gpc() ? stripslashes($value) : $value;
    }

    /**
     * @brief check that required fields are filled.
     *
     * it refers $vars[$key]['required']
     * @param[in] $file file object to be checked
     * @param[out] $missing array of string of missing field name
     * @retval ture filled
     * @retval false not filled
     */
    function isFilledRequired(&$missing) 
    {
        $missing = array();
        foreach(array_keys($this->vars) as $field) {
            if (!$this->vars[$field]['required']) continue;
            $var = &$this->get($field);
            if (is_array($var) && 0 == count($var) || !is_array($var) && empty($var)) {
                $missing[] = $field;
            }
        }
        return 0 == count($missing);
    }

    /**
     * @brief get a value of variable without any sanitizing
     * @param string $key name of the variable to get
     * @return mixed value of variable
     */
    function get($key) 
    {
        return $this->vars[$key]['value'];
    }

    /**
     * @brief set a sanitized value
     * @param string $key name of the variable to assign
     * @param  mixed value of variable to assin
     */
    function set($key, $value) 
    {
        $this->setVar($key, $value, true);
    }

    /**
     * get all variables(no format conversions)
     * @return array associative array of key->value pairs
     */
    function getArray() 
    {
        return $this->getVarArray('none');
    }

    /**
     * @brief compare to another object
     * @param obj XooNIpsTableObject or empty value
     * @return true if all vars equal to obj vars
     */
    function equals($obj) 
    {
        if (!$obj) {
            return false;
        }
        foreach(array_keys($this->vars) as $field) {
            if ($this->vars[$field]['value'] != $obj->get($field)) {
                return false;
            }
        }
        return true;
    }

    /**
     * assign a value to a variable
     *
     * @access public
     * @param string $key
     * @param string $value
     */
    function assignVar( $key, $value )
    {
        if (!empty($key) && isset($value) && isset($this->vars[$key])) {
            $this->vars[$key]['value'] =& $value;
        } else {
            $this->setExtraVar( $key, $value );
        }
    }

    /**
     * set extra var
     *
     * @access public
     * @param string $key
     * @param string $value
     */
    function setExtraVar( $key, $value )
    {
        return $this->_extra_vars[$key] =& $value;
    }

    /**
     * get extra var
     *
     * @access public
     * @param object $key
     * @return string extra var
     */
    function &getExtraVar( $key )
    {
        return $this->_extra_vars[$key];
    }
}

/**
 * The basic object handler class for the XooNIps
 */
class XooNIpsTableObjectHandler extends XoopsObjectHandler
{

    /**
     * a flag for initialized check
     * @var bool
     * @access private
     */
    var $__is_initialized = false;

    /**
     * class name of handling object
     * @var string
     * @access private
     */
    var $__class_name;

    /**
     * database table name for object mapping
     * @var string
     * @access private
     */
    var $__table_name;

    /**
     * primary key name of database table
     * @var string
     * @access private
     */
    var $__key_name;

    /**
     * flag for primary key is auto increment
     * @var bool
     * @access private
     */
    var $__is_autoincrement = true;

    /**
     * constructor
     *
     * normally, this is called from child classes only
     * @access public
     */
    function XooNIpsTableObjectHandler(&$db) 
    {
        $this->XoopsObjectHandler($db);
    }

    /**
     * initilizing function, this is called from child class only
     *
     * @access protected
     * @param string $cname class name
     * @param string $tname database table name for object mapping
     * @param string $key primary key name of database table
     * @param bool $is_autoincrement TRUE if primary key is autoincrement field
     */
    function __initHandler($cname, $tname, $key, $is_autoincrement = true) 
    {
        $this->__is_initialized = true;
        $this->__class_name = $cname;
        $this->__table_name = $tname;
        $this->__key_name = $key;
        $this->__is_autoincrement = $is_autoincrement;
    }

    /**
     * create a new object
     *
     * @access public
     * @param bool isNew mark the new object as 'new'?
     * @return object XooNIpsTableObject reference to the new object
     */
    function &create($isNew = true) 
    {
        if (!$this->__is_initialized) {
            $ret = false;
            return $ret;
        }
        $obj = &new $this->__class_name();
        if ($isNew) {
            $obj->setNew();
        }
        return $obj;
    }

    /**
     * gets a value object
     *
     * @access public
     * @param int $id
     * @return object XooNIpsTableObject reference to the object instance
     */
    function &get($id) 
    {
        $ret = false;
        if (!$this->__is_initialized) return $ret;
        if (intval($id) > 0) {
            $sql = sprintf('SELECT * FROM %s WHERE %s=%u', $this->db->prefix($this->__table_name) , $this->__key_name, $id);
            if ($result = $this->db->query($sql)) {
                $numrows = $this->db->getRowsNum($result);
                if ($numrows == 1) {
                    $obj = &new $this->__class_name();
                    $obj->assignVars($this->db->fetchArray($result));
                    $ret = &$obj;
                }
                $this->db->freeRecordSet($result);
            }
        }
        return $ret;
    }

    /**
     * insert/update object
     *
     * @access public
     * @param object $obj
     * @param bool $force operation
     * @return bool FALSE if failed
     */
    function insert(&$obj, $force = false) 
    {
        if (!$this->__is_initialized) return false;
        if (strtolower(get_class($obj)) != strtolower($this->__class_name)) {
            return false;
        }
        if (!$obj->isDirty()) {
            return true;
        }
        if (!$obj->cleanVars()) {
            return false;
        }
        if ($obj->isNew()) {
            $sql_arr = &$this->_makeVarsArray4SQL($obj, $obj->cleanVars);
            if ($this->__is_autoincrement) $myid = $this->db->genId($this->__table_name . '_' . $this->__key_name . '_seq');
            else $myid = $sql_arr[$this->__key_name];
            $sql_fields = array();
            $sql_values = array();
            foreach(array_keys($sql_arr) as $name) {
                $sql_fields[] = $name;
                if ($name == $this->__key_name) {
                    $sql_values[] = $myid;
                } else {
                    $sql_values[] = (is_null($sql_arr[$name]) ? 'NULL' : $sql_arr[$name]);
                }
            }
            $sql = sprintf("INSERT INTO %s ( %s ) VALUES ( %s )", $this->db->prefix($this->__table_name) , implode(",", $sql_fields) , implode(",", $sql_values));
        } else {
            $sql_arr = &$this->_makeVarsArray4SQL($obj, $obj->cleanVars);
            $myid = $sql_arr[$this->__key_name];
            $sql_keyl = array();
            $sql_setl = array();
            foreach(array_keys($sql_arr) as $name) {
                if ($name == $this->__key_name) {
                    $sql_keyl[] = $name . '=' . $sql_arr[$name];
                } else {
                    $sql_setl[] = $name . '=' . (is_null($sql_arr[$name]) ? 'NULL' : $sql_arr[$name]);
                }
            }
            $sql = sprintf("UPDATE %s SET %s WHERE %s", $this->db->prefix($this->__table_name) , implode(", ", $sql_setl) , implode(" AND ", $sql_keyl));
        }
        if ($force) {
            if (!$result = $this->db->queryF($sql)) {
                trigger_error($this->db->error());
                return false;
            }
        } else {
            if (!$result = $this->db->query($sql)) {
                trigger_error($this->db->error());
                return false;
            }
        }
        if (empty($myid)) {
            $myid = $this->db->getInsertId();
        }
        $obj->assignVar($this->__key_name, $myid);
        return true;
    }

    /**
     * helper function for sql string creation
     *
     * @access private
     * @param object $obj
     * @param array $vars array of variables
     * @return array quoted strings
     */
    function &_makeVarsArray4SQL(&$obj, &$vars) 
    {
        $ret = array();
        $keys_array = &$obj->getKeysArray();
        foreach($keys_array as $k) {
            switch ($obj->vars[$k]['data_type']) {
                case XOBJ_DTYPE_TXTBOX:
                case XOBJ_DTYPE_TXTAREA:
                case XOBJ_DTYPE_URL:
                case XOBJ_DTYPE_EMAIL:
                case XOBJ_DTYPE_ARRAY:
                case XOBJ_DTYPE_OTHER:
                case XOBJ_DTYPE_SOURCE:
                    if (is_null($vars[$k])) $ret[$k] = 'NULL';
                    else $ret[$k] = $this->db->quoteString($vars[$k]);
                    break;

                case XOBJ_DTYPE_INT:
                case XOBJ_DTYPE_STIME:
                case XOBJ_DTYPE_MTIME:
                case XOBJ_DTYPE_LTIME:
                default:
                    if (is_null($vars[$k])) $ret[$k] = 'NULL';
                    else $ret[$k] = $vars[$k];
                }
            }
            return $ret;
        }

        /**
         * delete object
         *
         * @access public
         * @param object $obj
         * @return bool FALSE if failed
         */
        function delete(&$obj) 
        {
            if (!$this->__is_initialized) return false;
            if (strtolower(get_class($obj)) != strtolower($this->__class_name)) {
                return false;
            }
            $sql = sprintf('DELETE FROM %s WHERE %s = %u', $this->db->prefix($this->__table_name) , $this->__key_name, $obj->get($this->__key_name));
            if (!$result = $this->db->query($sql)) {
                trigger_error($this->db->error());
                return false;
            }
            return true;
        }

        /**
         * gets objects
         *
         * @access public
         * @param object $criteria
         * @param bool $id_as_key
         * @param string $fieldlist fieldlist for distinct select
         * @param bool $distinct
         * @param XoopsJoinCriteria $joindef join criteria object
         * @return array objects
         */
        function &getObjects($criteria = null, $id_as_key = false, $fieldlist = '', $distinct = false, $joindef = null) 
        {
            if (!$this->__is_initialized) {
                $ret = false;
                return $ret;
            }
            $ret = array();
            $limit = $start = 0;
            $distinct = ($distinct) ? 'DISTINCT ' : '';
            $fieldlist = ($fieldlist == '') ? '*' : $fieldlist;
            $sql = sprintf('SELECT %s%s FROM %s', $distinct, $fieldlist, $this->db->prefix($this->__table_name));
            if ($joindef) {
                $sql.= $joindef->render($this->db->prefix($this->__table_name));
            }
            if (isset($criteria) && is_subclass_of($criteria, 'criteriaelement')) {
                $sql.= ' ' . $criteria->renderWhere();
            }
            if (isset($criteria) && (is_subclass_of($criteria, 'criteriaelement') || strtolower(get_class($criteria)) == 'criteriaelement')) {
                if ($criteria->getGroupby() != ' GROUP BY ') {
                    $sql.= ' ' . $criteria->getGroupby();
                }
                if ((is_array($criteria->getSort()) && count($criteria->getSort()) > 0)) {
                    $orderStr = 'ORDER BY ';
                    $orderDelim = "";
                    foreach($criteria->getSort() as $sortVar) {
                        $orderStr.= $orderDelim . $sortVar . ' ' . $criteria->getOrder();
                        $orderDelim = ",";
                    }
                    $sql.= ' ' . $orderStr;
                } else if ($criteria->getSort() != '') {
                    $orderStr = 'ORDER BY ' . $criteria->getSort() . ' ' . $criteria->getOrder();
                    $sql.= ' ' . $orderStr;
                }
                $limit = $criteria->getLimit();
                $start = $criteria->getStart();
            }
            $result = $this->db->query($sql, $limit, $start);
            if (!$result) {
                trigger_error($this->db->error());
                return $ret;
            }
            while ($myrow = $this->db->fetchArray($result)) {
                $obj = &new $this->__class_name();
                $obj->assignVars($myrow);
                if (!$id_as_key) {
                    $ret[] = &$obj;
                } else {
                    $ret[$myrow[$this->__key_name]] = &$obj;
                }
                unset($obj);
            }
            $this->db->freeRecordSet($result);
            return $ret;
        }

        /**
         * count how many rows in tables
         *
         * @access public
         * @param object $criteria
         * @return int number of rows
         */
        function getCount($criteria = null) 
        {
            $sql = sprintf('SELECT COUNT(*) FROM %s', $this->db->prefix($this->__table_name));
            if (isset($criteria) && is_subclass_of($criteria, 'criteriaelement')) {
                $sql.= ' ' . $criteria->renderWhere();
            }
            if (!$result = &$this->db->query($sql)) {
                trigger_error($this->db->error());
                return 0;
            }
            list($count) = $this->db->fetchRow($result);
            $this->db->freeRecordSet($result);
            return $count;
        }

        /**
         * delete objects using criteria
         *
         * @access public
         * @param object $criteria
         * @return bool FALSE if failed
         */
        function deleteAll($criteria = null) 
        {
            $sql = sprintf('DELETE FROM %s', $this->db->prefix($this->__table_name));
            if (isset($criteria) && is_subclass_of($criteria, 'criteriaelement')) {
                $sql.= ' ' . $criteria->renderWhere();
            }
            if (!$result = $this->db->query($sql)) {
                trigger_error($this->db->error());
                return false;
            }
            return true;
        }

        /**
         *
         * return primary key name
         * @return string primary key name
         */
        function getKeyName() 
        {
            return $this->__key_name;
        }
    }

    /**
     * Generic Table Manupulation XoopsObject class
     *
     * @copyright copyright (c) 2004-2006 Kowa.ORG
     * @author Nobuki Kowa <Nobuki@Kowa.ORG>
     * @package XoopsTableObject
     */
    class XoopsJoinCriteria
    {
        var $_table_name;
        var $_main_field;
        var $_sub_field;
        var $_join_type;
        var $_next_join;
        var $_table_alias; // thanks towdash
        function XoopsJoinCriteria($table_name, $main_field, $sub_field, $join_type = 'LEFT', $table_alias = "") 
        {
            $this->_table_name = $table_name;
            $this->_main_field = $main_field;
            $this->_sub_field = $sub_field;
            $this->_join_type = $join_type;
            $this->_next_join = false;
            $this->_next_join_main_table = false;
            $this->_table_alias = $table_alias;
        }
        function cascade(&$joinCriteria, $nextJoinMainTable = false) 
        {
            $this->_next_join = &$joinCriteria;
            $this->_next_join_main_table = $nextJoinMainTable;
        }
        function render($main_table) 
        {
            if ($this->_table_alias == "") {
                $table_alias = $this->_table_name;
                $alias_def = "";
            } else {
                $table_alias = $this->_table_alias;
                $alias_def = " AS " . $table_alias;
            }
            $join_str = " " . $this->_join_type . " JOIN " . $this->_table_name . $alias_def . " ON " . $main_table . "." . $this->_main_field . "=" . $table_alias . "." . $this->_sub_field . " ";
            if ($this->_next_join) {
                if ($this->_next_join_main_table) {
                    $join_str.= $this->_next_join->render($this->_next_join_main_table);
                } else {
                    $join_str.= $this->_next_join->render($table_alias);
                }
            }
            return $join_str;
        }
        function getMainAlias() 
        {
            return $this->_main_alias;
        }
    }
?>
