<?php
// $Revision: 1.1.2.4 $
//  ------------------------------------------------------------------------ //
//  XooNIps - Neuroinformatics Base Platform System                          //
//  Copyright (C) 2005-2007 RIKEN, Japan All rights reserved.                //
//  http://xoonips.sourceforge.jp/                                           //
//  ------------------------------------------------------------------------ //
//  This program is free software; you can redistribute it and/or modify     //
//  it under the terms of the GNU General Public License as published by     //
//  the Free Software Foundation; either version 2 of the License, or        //
//  (at your option) any later version.                                      //
//                                                                           //
//  You may not change or alter any portion of this comment or credits       //
//  of supporting developers from this source code or any supporting         //
//  source code which is considered copyrighted (c) material of the          //
//  original comment or credit authors.                                      //
//                                                                           //
//  This program is distributed in the hope that it will be useful,          //
//  but WITHOUT ANY WARRANTY; without even the implied warranty of           //
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            //
//  GNU General Public License for more details.                             //
//                                                                           //
//  You should have received a copy of the GNU General Public License        //
//  along with this program; if not, write to the Free Software              //
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA //
//  ------------------------------------------------------------------------ //

include_once XOOPS_ROOT_PATH . '/modules/xoonips/class/base/transaction.class.php';
include_once XOOPS_ROOT_PATH . '/modules/xoonips/class/base/logic.class.php';
include_once XOOPS_ROOT_PATH . '/modules/xoonips/class/base/transaction.class.php';

/**
 *
 * subclass of XooNIpsLogic(login)
 *
 */
class XooNIpsLogicLogin extends XooNIpsLogic
{

    /**
     * execute login
     *
     * @param[in] $vars[0] id (use '' if guest login)
     * @param[in] $vars[1] pass (use '' if guest login)
     * @param[out] $response->result true:success, false:failed
     * @param[out] $response->error error information
     * @param[out] $response->success session id
     * @return false if error
     */
    function execute(&$vars, &$response) 
    {
        global $xoopsConfig;
        // parameter check
        $error = &$response->getError();
        if (count($vars) > 2) $error->add(XNPERR_EXTRA_PARAM);
        if (count($vars) < 2) $error->add(XNPERR_MISSING_PARAM);
        //
        if (isset($vars[0]) && strlen($vars[0]) > 25) $error->add(XNPERR_INVALID_PARAM, 'too long parameter 1');
        //
        if ($error->get(0)) {
            // return if parameter error
            $response->setResult(false);
            return;
        } else {
            $id = $vars[0];
            $pass = $vars[1];
        }
        $member_handler = &xoonips_gethandler('member');
        $user_handler = &xoonips_getormhandler('xoonips', 'users');
        $event_log_handler = &xoonips_getormhandler('xoonips', 'event_log');
        $config_handler = &xoonips_getormhandler('xoonips', 'config');
        $transaction = XooNIpsTransaction::getInstance();
        $transaction->start();
        if ($id == '') {
            $target_user = &$config_handler->get(XNP_CONFIG_PUBLIC_ITEM_TARGET_USER_KEY);
            if ($pass != '' || $target_user->get('value') != XNP_CONFIG_PUBLIC_ITEM_TARGET_USER_ALL) {
                $transaction->rollback();
                $response->error->add(XNPERR_AUTH_FAILURE);
                $response->setResult(false);
                return false;
            }
            $user = false;
            $uid = UID_GUEST;
            $groups = array();
        } else {
            $user = &$member_handler->loginUser($id, $pass);
            if (!$user) {
                $transaction->rollback();
                // insert login failure event
                $event_log = &$event_log_handler->create();
                $event_log->setVar('event_type_id', ETID_LOGIN_FAILURE);
                $event_log->setVar('timestamp', time());
                $event_log->setVar('exec_uid', UID_GUEST);
                $event_log->setVar('remote_host', xnpGetRemoteHost());
                $event_log->setVar('additional_info', $id);
                if (!$event_log_handler->insert($event_log)) {
                    $response->error->add(XNPERR_SERVER_ERROR, "cannot insert event");
                }
                // return error
                $response->error->add(XNPERR_AUTH_FAILURE);
                $response->setResult(false);
                return false;
            }
            $xoonips_user = $user->getVar('xoonips_user');
            $uid = $xoonips_user->get('uid');
            $xoops_user_handler = xoops_gethandler('user');
            $xoops_user = $xoops_user_handler->get($uid);
            if (0 == $xoops_user->getVar('level', 'n') || !$xoonips_user->get('activate')) { // not activated, not certified
                // return error
                $transaction->rollback();
                $response->error->add(XNPERR_AUTH_FAILURE);
                $response->setResult(false);
                return false;
            }
            $groups = $xoops_user->getGroups();
        }
        if ($xoopsConfig['closesite'] == 1) {
            $allowed = false;
            if ($user) {
                foreach($groups as $group) {
                    if (in_array($group, $xoopsConfig['closesite_okgrp']) || XOOPS_GROUP_ADMIN == $group) {
                        $allowed = true;
                        break;
                    }
                }
            }
            if (!$allowed) {
                // site closed
                $transaction->rollback();
                $response->error->add(XNPERR_AUTH_FAILURE);
                $response->setResult(false);
                return false;
            }
        }
        // remove expired xoonips sessions
        $expire_time = time() +(60*$xoopsConfig['session_expire']);
        $session_handler = &xoonips_getormhandler('xoonips', 'session');
        $join = new XoopsJoinCriteria($GLOBALS['xoopsDB']->prefix('session') , 'sess_id', 'sess_id', 'left', 'ts');
        $criteria = new Criteria('isnull(ts.sess_id)', 1);
        $sessions = $session_handler->getObjects($criteria, false, $GLOBALS['xoopsDB']->prefix('xoonips_session') . '.*', false, $join);
        foreach($sessions as $session) {
            if ($session->get('su_uid')) {
                // insert end_su event
                $event_log = &$event_log_handler->create();
                $event_log->setVar('event_type_id', ETID_END_SU);
                $event_log->setVar('timestamp', $expire_time);
                $event_log->setVar('exec_uid', $session->get('uid'));
                $event_log->setVar('remote_host', '');
                $event_log->setVar('uid', $session->get('su_uid'));
                if (!$event_log_handler->insert($event_log)) {
                    $transaction->rollback();
                    $response->error->add(XNPERR_SERVER_ERROR, "cannot insert event log");
                    $response->setResult(false);
                    return false;
                }
            }
            // insert logout event
            $event_log = &$event_log_handler->create();
            $event_log->setVar('event_type_id', ETID_LOGOUT);
            $event_log->setVar('timestamp', $expire_time);
            $event_log->setVar('exec_uid', $session->get('uid'));
            $event_log->setVar('remote_host', '');
            if (!$event_log_handler->insert($event_log)) {
                $transaction->rollback();
                $response->error->add(XNPERR_SERVER_ERROR, "cannot insert event log");
                $response->setResult(false);
                return false;
            }
            if (!$session_handler->delete($session)) {
                $transaction->rollback();
                $response->error->add(XNPERR_SERVER_ERROR, "cannot delete old session");
                $response->setResult(false);
                return false;
            }
        }
        // record $uid
        $_SESSION = array();
        $_SESSION['xoopsUserId'] = $uid;
        $_SESSION['xoopsUserGroups'] = $groups;
        
        // set XNPSID(for old routines)
        $_SESSION['XNPSID'] = ( $uid == UID_GUEST ) ? SID_GUEST : session_id();
        
        if ($user) {
            // update last_login
            $xoops_user->setVar('last_login', time());
            if (!$xoops_user_handler->insert($xoops_user)) {
            }
            // insert login event
            $event_log = &$event_log_handler->create();
            $event_log->setVar('event_type_id', ETID_LOGIN_SUCCESS);
            $event_log->setVar('timestamp', time());
            $event_log->setVar('exec_uid', $uid);
            $event_log->setVar('remote_host', xnpGetRemoteHost());
            $event_log_handler->insert($event_log);
            // insert xoonips_session
            $session = &$session_handler->create();
            $session->setVar('sess_id', session_id());
            $session->setVar('uid', $uid);
            $session->setVar('updated', time());
            $session_handler->insert($session);
        }
        $transaction->commit();
        $response->setSuccess(session_id());
        $response->setResult(true);
        return true;
    }
}
?>
