<?php
// $Revision: 1.1.2.5 $
//  ------------------------------------------------------------------------ //
//  XooNIps - Neuroinformatics Base Platform System                          //
//  Copyright (C) 2005-2007 RIKEN, Japan All rights reserved.                //
//  http://xoonips.sourceforge.jp/                                           //
//  ------------------------------------------------------------------------ //
//  This program is free software; you can redistribute it and/or modify     //
//  it under the terms of the GNU General Public License as published by     //
//  the Free Software Foundation; either version 2 of the License, or        //
//  (at your option) any later version.                                      //
//                                                                           //
//  You may not change or alter any portion of this comment or credits       //
//  of supporting developers from this source code or any supporting         //
//  source code which is considered copyrighted (c) material of the          //
//  original comment or credit authors.                                      //
//                                                                           //
//  This program is distributed in the hope that it will be useful,          //
//  but WITHOUT ANY WARRANTY; without even the implied warranty of           //
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            //
//  GNU General Public License for more details.                             //
//                                                                           //
//  You should have received a copy of the GNU General Public License        //
//  along with this program; if not, write to the Free Software              //
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA //
//  ------------------------------------------------------------------------ //

include_once XOOPS_ROOT_PATH . '/modules/xoonips/class/base/logic.class.php';

/**
 *
 * subclass of XooNIpsLogic(searchItem)
 *
 */
class XooNIpsLogicSearchItem extends XooNIpsLogic
{
    function getSearchCacheIdByQuery($query) 
    {
        if (isset($_SESSION['xoonips_search_cache']) && isset($_SESSION['xoonips_search_cache']["_$query"])) {
            return $_SESSION['xoonips_search_cache']["_$query"];
        }
        return false;
    }
    function putSearchCache($query, $search_cache_id) 
    {
        if (!isset($_SESSION['xoonips_search_cache'])) {
            $_SESSION['xoonips_search_cache'] = array();
        }
        $_SESSION['xoonips_search_cache']["_$query"] = $search_cache_id;
    }

    /**
     * execute searchItem
     *
     * @param[in]  $vars[0] session ID
     * @param[in]  $vars[1] query string
     * @param[in]  $vars[2] offset of the first row to return (The offset of the initial row is 0)
     * @param[in]  $vars[3] maximum number of rows to return (0:all rows)
     * @param[in]  $vars[4] sort key('title'|'ext_id'|'last_modified_date'|'registration_date'|'creation_date')
     * @param[in]  $vars[5] sort order('asc'|'desc')
     * @param[out] $response->result true:success, false:failed
     * @param[out] $response->error  error information
     * @param[out] $response->success['count'] number of matched item
     * @param[out] $response->success[n] search result
     * @param[out] $response->success[n]['id'] item id of matched item
     * @param[out] $response->success[n]['matchfor'] bit flag represents what part of item matched
     */
    function execute(&$vars, &$response) 
    {
        // parameter check
        $error = &$response->getError();
        if (count($vars) > 6) $error->add(XNPERR_EXTRA_PARAM);
        if (count($vars) < 6) $error->add(XNPERR_MISSING_PARAM);
        //
        if (isset($vars[0]) && strlen($vars[0]) > 32) $error->add(XNPERR_INVALID_PARAM, 'too long parameter 1');
        $ar = explode(':', $vars[1]);
        if (count($ar) < 2 || !in_array($ar[0], array(
            'index',
            'keyword'
        ))) $error->add(XNPERR_INVALID_PARAM, 'bad query parameter 2');
        if (!is_int($vars[2]) && !ctype_digit($vars[2])) $error->add(XNPERR_INVALID_PARAM, 'not integer parameter 2');
        if (!is_int($vars[3]) && !ctype_digit($vars[3])) $error->add(XNPERR_INVALID_PARAM, 'not integer parameter 3');
        if (!in_array($vars[4], array(
            'title',
            'ext_id',
            'last_modified_date',
            'registration_date',
            'creation_date'
        ))) $error->add(XNPERR_INVALID_PARAM, 'unknown sort key parameter 4');
        if (!in_array($vars[5], array(
            'asc',
            'desc'
        ))) $error->add(XNPERR_INVALID_PARAM, 'unknown sort order parameter 5');
        //
        if ($error->get(0)) {
            // return if parameter error
            $response->setResult(false);
            return;
        } else {
            $sessionid = $vars[0];
            $query = $vars[1];
            $start = $vars[2];
            $limit = $vars[3];
            $sortkey = $vars[4];
            $ascdesc = $vars[5];
        }
        list($result, $uid, $session) = $this->restoreSession($sessionid);
        if (!$result) {
            $response->setResult(false);
            $error->add(XNPERR_INVALID_SESSION);
            return false;
        }
        $response->setResult(false);
        $search_cache_id = $this->getSearchCacheIdByQuery($query);
        //        $search_cache_id = false;
        if (!$search_cache_id) {
            $search_cache_handler = &xoonips_getormhandler('xoonips', 'search_cache');
            $search_cache = $search_cache_handler->create();
            $search_cache->setVar('sess_id', $sessionid);
            if (!$search_cache_handler->insert($search_cache)) {
                $response->setResult(false);
                $error->add(XNPERR_SERVER_ERROR, "cannot create search_cache_id");
                return false;
            }
            $search_cache_handler->insert($search_cache);
            $search_cache_id = $search_cache->get('search_cache_id');
            if (substr($query, 0, 6) == 'index:') {
                $this->indexSearch($error, $search_cache_id, $uid, substr($query, 6));
            } else if (substr($query, 0, 8) == 'keyword:') {
                $this->keywordSearch($error, $search_cache_id, $uid, substr($query, 8));
                $event_log_handler = &xoonips_getormhandler('xoonips', 'event_log');
                $event_log = $event_log_handler->create();
                $event_log->setVar('event_type_id', ETID_QUICK_SEARCH );
                $event_log->setVar('timestamp', time());
                $event_log->setVar('exec_uid', $uid);
                $event_log->setVar('remote_host', xnpGetRemoteHost());
                $event_log->setVar('search_keyword', 'search_itemtype=all&keyword=' . urlencode(substr($query, 8)) );
                if ( !$event_log_handler->insert( $event_log ) ){
                    $error->add(XNPERR_SERVER_ERROR, "cannot insert event");
                }
            } else {
                $error->add(XNPERR_INVALID_PARAM, "query must begin with index: or keyword:");
            }
            if ($error->get()) {
                $response->setResult(false);
                return false;
            }
            $this->putSearchCache($query, $search_cache_id);
        }
        //
        // access permission: ol_public && ( is_moderator || certify_state=certified) || ol_group_only && gid=gulink.gid && ( is_admin || certify_state==certified) || ol_private && uid=uid // how to reuse?
        $criteria = new CriteriaCompo();
        $criteria->add(new Criteria('open_level', OL_PUBLIC));
        $member_handler = &xoonips_gethandler('member');
        if (!$member_handler->isModerator($uid)) {
            $criteria->add(new Criteria('certify_state', CERTIFIED));
        }
        $groups_users_link_handler = &xoonips_getormhandler('xoonips', 'groups_users_link');
        $links = $groups_users_link_handler->getObjects(new Criteria('uid', $uid));
        foreach($links as $link) {
            $criteria->add(new Criteria('open_level', OL_GROUP_ONLY) , 'OR');
            $criteria->add(new Criteria('gid', $link->get('gid')));
            if (!$link->get('is_admin')) {
                $criteria->add(new Criteria('certify_state', CERTIFIED));
            }
        }
        $criteria->add(new Criteria('open_level', OL_PRIVATE) , 'OR');
        $criteria->add(new Criteria('tx.uid', $uid));
        $criteria = new CriteriaCompo($criteria);
        $criteria->add(new Criteria('search_cache_id', $search_cache_id));
        $criteria->setOrder($ascdesc);
        $criteria->setStart($start);
        $criteria->setLimit($limit);
        $criteria->setGroupby('search_cache_item_id');
        switch ($sortkey) {
            case 'title':
                $join_sort_table = 'xoonips_item_title';
                $criteria->add(new Criteria('title_id', 0));
                $criteria->setSort('title');
                break;

            case 'ext_id':
                $join_sort_table = 'xoonips_item_basic';
                $criteria->setSort('doi');
                break;

            case 'last_modified_date':
                $join_sort_table = 'xoonips_item_basic';
                $criteria->setSort('last_update_date');
                break;

            case 'registration_date':
                $join_sort_table = 'xoonips_item_basic';
                $criteria->setSort('creation_date');
                break;

            case 'creation_date':
                $join_sort_table = 'xoonips_item_basic';
                $criteria->setSort(array(
                    'publication_year',
                    'publication_month',
                    'publication_mday'
                ));
                break;

            default:
                $error->add(XNPERR_INVALID_PARAM, 'unknown sort key parameter 4');
                $response->setResult(false);
                return false;
                break;
        }
        $join = new XoopsJoinCriteria($GLOBALS['xoopsDB']->prefix('xoonips_index_item_link') , 'item_id', 'item_id', 'left', 'txil');
        $join2 = new XoopsJoinCriteria($GLOBALS['xoopsDB']->prefix('xoonips_index') , 'index_id', 'index_id', 'left', 'tx');
        $join3 = new XoopsJoinCriteria($GLOBALS['xoopsDB']->prefix($join_sort_table) , 'item_id', 'item_id');
        $join->cascade($join2);
        $join2->cascade($join3, $GLOBALS['xoopsDB']->prefix('xoonips_search_cache_item'));
        $search_cache_item_handler = &xoonips_getormhandler('xoonips', 'search_cache_item');
        $search_cache_items = $search_cache_item_handler->getObjects($criteria, false, '', false, $join);
        if ($search_cache_items === false) {
            $error->add(XNPERR_INVALID_PARAM, 'cannot get searched item');
            $response->setResult(false);
            return false;
        }
        $response->setSuccess($search_cache_items);
        $response->setResult(true);
        return true;
    }
    function keywordSearch(&$error, $search_cache_id, $uid, $query) 
    {
        $search_cache_item_handler = &xoonips_getormhandler('xoonips', 'search_cache_item');
        // search item
        $item_type_handler = &xoonips_getormhandler('xoonips', 'item_type');
        $item_types = $item_type_handler->getObjects(new Criteria('isnull(mid)', 0));
        //$item_types = $item_type_handler->getObjects( new Criteria( 'mid', 20 ) ); // todo: 20:book
        foreach($item_types as $item_type) {
            $detail_item_handler = &xoonips_getormcompohandler($item_type->get('name') , 'item');
            if (!$detail_item_handler) {
                continue;
            }
            $step = 1000;
            for ($offset = 0;; $offset+= $step) {
                $item_ids = $detail_item_handler->search($query, $step, $offset, $uid);
                if ($item_ids === false) {
                    $error->add(XNPERR_SERVER_ERROR, "cannot search item");
                    return;
                }
                foreach($item_ids as $item_id) {
                    $search_cache_item = $search_cache_item_handler->create();
                    $search_cache_item->setVar('search_cache_id', $search_cache_id);
                    $search_cache_item->setVar('item_id', $item_id);
                    $search_cache_item->setVar('matchfor_index', 0);
                    $search_cache_item->setVar('matchfor_item', 1);
                    $search_cache_item->setVar('matchfor_file', 0);
                    if (!$search_cache_item_handler->insert($search_cache_item)) {
                        $error->add(XNPERR_SERVER_ERROR, "cannot get items");
                        return;
                    }
                }
                if (count($item_ids) != $step) {
                    break;
                }
            }
        }
        // search file
        $step = 1000;
        $search_text_handler = &xoonips_getormhandler('xoonips', 'search_text');
        $file_handler = &xoonips_getormhandler('xoonips', 'file');
        for ($offset = 0;; $offset+= $step) {
            $item_ids = $search_text_handler->search($query, $step, $offset, $uid);
            if ($item_ids === false) {
                $error->add(XNPERR_SERVER_ERROR, "cannot search file");
                return;
            }
            if (count($item_ids)) {
                // update search_cache_item
                $criteria = new CriteriaCompo();
                $criteria->add(new Criteria('item_id', '(' . implode(',', $item_ids) . ')', 'in'));
                $criteria->add(new Criteria('search_cache_id', $search_cache_id));
                $search_cache_items = $search_cache_item_handler->getObjects($criteria);
                if ($search_cache_items === false) {
                    $error->add(XNPERR_SERVER_ERROR, "cannot get search_cache_item");
                    return;
                }
                $len = count($search_cache_items);
                $update_item_ids = array();
                for ($i = 0; $i < $len; $i++) {
                    $search_cache_items[$i]->setVar('matchfor_file', 1);
                    if (!$search_cache_item_handler->insert($search_cache_items[$i])) {
                        $error->add(XNPERR_SERVER_ERROR, "cannot update search_cache_item");
                        return;
                    }
                    $update_item_ids[] = $search_cache_items[$i]->get('item_id');
                }
                // insert search_cache_item
                $insert_item_ids = array_diff($item_ids, $update_item_ids);
                foreach($insert_item_ids as $item_id) {
                    $search_cache_item = $search_cache_item_handler->create();
                    $search_cache_item->setVar('search_cache_id', $search_cache_id);
                    $search_cache_item->setVar('item_id', $item_id);
                    $search_cache_item->setVar('matchfor_index', 0);
                    $search_cache_item->setVar('matchfor_item', 0);
                    $search_cache_item->setVar('matchfor_file', 1);
                    if (!$search_cache_item_handler->insert($search_cache_item)) {
                        $error->add(XNPERR_SERVER_ERROR, "cannot insert search_cache_item");
                        return;
                    }
                }
            }
            if (count($item_ids) != $step) {
                break;
            }
        }
        return;
    }
    function indexSearch(&$error, $search_cache_id, $uid, $index_id) 
    {
        if (!ctype_digit($index_id)) {
            $error->add(XNPERR_INVALID_PARAM, "bad index id(index_id=$index_id)");
            return;
        }
        $criteria = new CriteriaCompo();
        $criteria->add(new Criteria('index_id', $index_id));
        $index_handler = &xoonips_getormhandler('xoonips', 'index');
        $index = $index_handler->get($index_id);
        if (false == $index){
            $error->add(XNPERR_NOT_FOUND, "index not found(index_id=$index_id)");
            return;
        }
        if (!$index_handler->getPerm($index_id, $uid, 'read')) {
            $error->add(XNPERR_ACCESS_FORBIDDEN, "forbidden(index_id=$index_id)");
            return;
        }
        if (!$index_handler->getPerm($index_id, $uid, 'admin')) {
            $criteria->add(new Criteria('certify_state', CERTIFIED));
        }
        $index_item_link_handler = &xoonips_getormhandler('xoonips', 'index_item_link');
        $index_item_links = $index_item_link_handler->getObjects($criteria);
        $search_cache_item_handler = &xoonips_getormhandler('xoonips', 'search_cache_item');
        foreach($index_item_links as $index_item_link) {
            $search_cache_item = $search_cache_item_handler->create();
            $search_cache_item->setVar('search_cache_id', $search_cache_id);
            $search_cache_item->setVar('item_id', $index_item_link->get('item_id'));
            $search_cache_item->setVar('matchfor_index', 1);
            $search_cache_item->setVar('matchfor_item', 0);
            $search_cache_item->setVar('matchfor_file', 0);
            if (!$search_cache_item_handler->insert($search_cache_item)) {
                $error->add(XNPERR_SERVER_ERROR, "cannot insert search_cache_item");
                return;
            }
        }
        return;
    }
}
?>
