<?php
// $Revision: 1.1.2.8 $
//  ------------------------------------------------------------------------ //
//  XooNIps - Neuroinformatics Base Platform System                          //
//  Copyright (C) 2005-2007 RIKEN, Japan All rights reserved.                //
//  http://xoonips.sourceforge.jp/                                           //
//  ------------------------------------------------------------------------ //
//  This program is free software; you can redistribute it and/or modify     //
//  it under the terms of the GNU General Public License as published by     //
//  the Free Software Foundation; either version 2 of the License, or        //
//  (at your option) any later version.                                      //
//                                                                           //
//  You may not change or alter any portion of this comment or credits       //
//  of supporting developers from this source code or any supporting         //
//  source code which is considered copyrighted (c) material of the          //
//  original comment or credit authors.                                      //
//                                                                           //
//  This program is distributed in the hope that it will be useful,          //
//  but WITHOUT ANY WARRANTY; without even the implied warranty of           //
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            //
//  GNU General Public License for more details.                             //
//                                                                           //
//  You should have received a copy of the GNU General Public License        //
//  along with this program; if not, write to the Free Software              //
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA //
//  ------------------------------------------------------------------------ //

include_once XOOPS_ROOT_PATH . "/modules/xoonips/class/base/tableobject.class.php";

/**
 * @brief Data object of XooNIps event
 *
 * @li getVar('event_id') :
 * @li getVar('event_type_id') :
 * @li getVar('timestamp') :
 * @li getVar('exec_uid') :
 * @li getVar('remote_host') :
 * @li getVar('index_id') :
 * @li getVar('item_id') :
 * @li getVar('file_id') :
 * @li getVar('uid') :
 * @li getVar('gid') :
 * @li getVar('search_keyword') :
 * @li getVar('additional_info') :
 *
 */
class XooNIpsEventLog extends XooNIpsTableObject
{
    function XooNIpsEventLog() 
    {
        parent::XooNIpsTableObject();
        $this->initVar('event_id', XOBJ_DTYPE_INT, null, false, null);
        $this->initVar('event_type_id', XOBJ_DTYPE_INT, null, true, null);
        $this->initVar('timestamp', XOBJ_DTYPE_INT, null, true, null);
        $this->initVar('exec_uid', XOBJ_DTYPE_INT, null, false, null);
        $this->initVar('remote_host', XOBJ_DTYPE_TXTBOX, null, false, 255);
        $this->initVar('index_id', XOBJ_DTYPE_INT, null, false, null);
        $this->initVar('item_id', XOBJ_DTYPE_INT, null, false, null);
        $this->initVar('file_id', XOBJ_DTYPE_INT, null, false, null);
        $this->initVar('uid', XOBJ_DTYPE_INT, null, false, null);
        $this->initVar('gid', XOBJ_DTYPE_INT, null, false, null);
        $this->initVar('search_keyword', XOBJ_DTYPE_TXTBOX, null, false, 65536);
        $this->initVar('additional_info', XOBJ_DTYPE_TXTBOX, null, false, 65536);
    }

    function cleanVars() 
    {
        $event_type_id = $this->get('event_type_id');
        if ($event_type_id < 1 || ETID_MAX < $event_type_id) {
            $result = false;
            $this->setErrors("invalid event_type_id(${event_type_id})");
        } else {
            $result = true;
            static $eventValidFields = array(array(0, 0, 0, 0, 0, 0, 0, 0, 0), //(dummy)
            array(1, 1, 0, 0, 0, 0, 0, 0, 1), //LOGIN_FAILURE
            array(1, 1, 0, 0, 0, 0, 0, 0, 0), //LOGIN_SUCCESS
            array(1, 1, 0, 0, 0, 0, 0, 0, 0), //LOGOUT
            array(1, 1, 0, 1, 0, 0, 0, 0, 0), //INSERT_ITEM
            array(1, 1, 0, 1, 0, 0, 0, 0, 0), //UPDATE_ITEM
            array(1, 1, 0, 1, 0, 0, 0, 0, 0), //DELETE_ITEM
            array(1, 1, 0, 1, 0, 0, 0, 0, 0), //VIEW_ITEM
            array(1, 1, 0, 1, 1, 0, 0, 0, 0), //DOWNLOAD_FILE
            array(1, 1, 1, 1, 0, 0, 0, 0, 0), //REQUEST_CERTIFY_ITEM
            array(1, 1, 1, 0, 0, 0, 0, 0, 0), //INSERT_INDEX
            array(1, 1, 1, 0, 0, 0, 0, 0, 0), //UPDATE_INDEX
            array(1, 1, 1, 0, 0, 0, 0, 0, 0), //DELETE_INDEX
            array(1, 1, 1, 1, 0, 0, 0, 0, 0), //CERTIFY_ITEM
            array(1, 1, 1, 1, 0, 0, 0, 0, 0), //REJECT_ITEM
            array(1, 0, 0, 0, 0, 1, 0, 0, 0), //REQUEST_INSERT_ACCOUNT
            array(1, 1, 0, 0, 0, 1, 0, 0, 0), //CERTIFY_ACCOUNT
            array(1, 1, 0, 0, 0, 0, 1, 0, 0), //INSERT_GROUP
            array(1, 1, 0, 0, 0, 0, 1, 0, 0), //UPDATE_GROUP
            array(1, 1, 0, 0, 0, 0, 1, 0, 0), //DELETE_GROUP
            array(1, 1, 0, 0, 0, 1, 1, 0, 0), //INSERT_GROUP_MEMBER
            array(1, 1, 0, 0, 0, 1, 1, 0, 0), //DELETE_GROUP_MEMBER
            array(1, 1, 0, 0, 0, 0, 0, 0, 0), //VIEW_TOP_PAGE
            array(1, 1, 0, 0, 0, 0, 0, 1, 0), //QUICK_SEARCH
            array(1, 1, 0, 0, 0, 0, 0, 1, 0), //ADVANCED_SEARCH
            array(1, 1, 0, 0, 0, 1, 0, 0, 0), //ETID_START_SU
            array(1, 1, 0, 0, 0, 1, 0, 0, 0), //ETID_END_SU
            );
            $fields = array('timestamp', 'exec_uid', 'index_id', 'item_id', 'file_id', 'uid', 'gid', 'search_keyword', 'additional_info',);
            foreach($fields as $i => $field) {
                // check unnecessary && specified values
                if ($eventValidFields[$event_type_id][$i] == 0 && !is_null($this->vars[$field]['value'])) {
                    $result = false;
                    $this->setErrors("cannot specify $field if event_type_id=$event_type_id");
                }
                // check necessary && unspecified values
                $this->vars[$field]['required'] = (bool)($eventValidFields[$event_type_id][$i]);
            }
        }
        return $result&parent::cleanVars();
    }
}

/**
 * @brief Handler object of XooNIps event
 *
 *
 */
class XooNIpsEventLogHandler extends XooNIpsTableObjectHandler
{
    function XooNIpsEventLogHandler(&$db) 
    {
        parent::XooNIpsTableObjectHandler($db);
        $this->__initHandler('XooNIpsEventLog', 'xoonips_event_log', 'event_id');
    }

    /**
     * create a new object
     *
     * @access public
     * @param bool isNew mark the new object as 'new'?
     * @return object XooNIpsEventLog reference to the new object
     */
    function &create( $isNew = true )
    {
        $obj = parent::create( $isNew );
        if ( $obj === false ) {
            return $obj;
        }
        if ( $isNew ) {
            $obj->setVar( 'remote_host', $this->getRemoteHost(), true ); // not gpc
        }
        return $obj;
    }

    /**
     * insert/update object
     *
     * @access public
     * @param object $obj
     * @param bool $force operation
     * @return bool FALSE if failed
     */
    function insert( &$obj, $force = false )
    {
        $obj->setVar( 'timestamp', time(), true ); // not gpc
        return parent::insert( $obj, $force );
    }

    /**
     * get remote host
     *
     * @access private
     * @return string remote host name or address
     */
    function getRemoteHost()
    {
        $remote_host = '';
        if ( isset( $_SERVER['REMOTE_HOST'] ) ) {
            $remote_host = $_SERVER['REMOTE_HOST'];
        } else if ( isset( $_SERVER['REMOTE_ADDR'] ) ) {
            $remote_host = $_SERVER['REMOTE_ADDR'];
        }
        // check proxy environment
        if ( isset( $_SERVER['HTTP_X_FORWARDED_FOR'] ) ) {
            $remote_host = $_SERVER['HTTP_X_FORWARDED_FOR'];
        } else if ( isset( $_SERVER['HTTP_VIA'] ) ) {
            $remote_host = $_SERVER['HTTP_VIA'];
        }
        if ( preg_match( '/^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}$/', $remote_host ) ) {
            if ( function_exists( 'gethostbyaddr' ) ) {
                $remote_host = gethostbyaddr( $remote_host );
            }
        }
        return $remote_host;
    }

    /**
     * get execution user id
     *
     * @access private
     * @return int execution user id
     */
    function getExecUid()
    {
        $exec_uid = UID_GUEST;
        if ( isset( $GLOBALS['xoopsUser'] ) && is_object( $GLOBALS['xoopsUser'] ) ) {
            $exec_uid = intval( $GLOBALS['xoopsUser']->getVar( 'uid', 'n' ) );
        }
        return $exec_uid;
    }

    /**
     * record insert item event (ETID_INSERT_ITEM: 4)
     *
     * @access public
     * @param int item_id insertion item id
     * @return bool false if failure
     */
    function recordInsertItemEvent( $item_id )
    {
        $obj =& $this->create();
        $obj->setVar( 'event_type_id', ETID_INSERT_ITEM, true ); // not gpc
        $obj->setVar( 'exec_uid', $this->getExecUid(), true ); // not gpc
        $obj->setVar( 'item_id', $item_id, true ); // not gpc
        return $this->insert( $obj );
    }

    /**
     * record update item event (ETID_UPDATE_ITEM: 5)
     *
     * @access public
     * @param int item_id updating item id
     * @return bool false if failure
     */
    function recordUpdateItemEvent( $item_id )
    {
        $obj =& $this->create();
        $obj->setVar( 'event_type_id', ETID_UPDATE_ITEM, true ); // not gpc
        $obj->setVar( 'exec_uid', $this->getExecUid(), true ); // not gpc
        $obj->setVar( 'item_id', $item_id, true ); // not gpc
        return $this->insert( $obj );
    }

    /**
     * record request certify item event (ETID_CERTIFY_ITEM: 9)
     *
     * @access public
     * @param int item_id insertion item id
     * @param int index_id insertion index id
     * @return bool false if failure
     */
    function recordRequestCertifyItemEvent( $item_id, $index_id )
    {
        $obj =& $this->create();
        $obj->setVar( 'event_type_id', ETID_REQUEST_CERTIFY_ITEM, true ); // not gpc
        $obj->setVar( 'exec_uid', $this->getExecUid(), true ); // not gpc
        $obj->setVar( 'item_id', $item_id, true ); // not gpc
        $obj->setVar( 'index_id', $index_id, true ); // not gpc
        return $this->insert( $obj );
    }

    /**
     * record insert index event (ETID_INSERT_INDEX: 10)
     *
     * @access public
     * @param int index_id insertion index id
     * @return bool false if failure
     */
    function recordInsertIndexEvent( $index_id )
    {
        $obj =& $this->create();
        $obj->setVar( 'event_type_id', ETID_INSERT_INDEX, true ); // not gpc
        $obj->setVar( 'exec_uid', $this->getExecUid(), true ); // not gpc
        $obj->setVar( 'index_id', $index_id, true ); // not gpc
        return $this->insert( $obj );
    }

    /**
     * record certify item event (ETID_CERTIFY_ITEM: 13)
     *
     * @access public
     * @param int item_id insertion item id
     * @param int index_id insertion index_id
     * @return bool false if failure
     */
    function recordCertifyItemEvent( $item_id, $index_id )
    {
        $obj =& $this->create();
        $obj->setVar( 'event_type_id', ETID_CERTIFY_ITEM, true ); // not gpc
        $obj->setVar( 'exec_uid', $this->getExecUid(), true ); // not gpc
        $obj->setVar( 'item_id', $item_id, true ); // not gpc
        $obj->setVar( 'index_id', $index_id, true ); // not gpc
        return $this->insert( $obj );
    }

    /**
     * record request insert account event (ETID_REQUEST_INSERT_ACCOUNT: 15)
     *
     * @access public
     * @param int uid insertion user id
     * @return bool false if failure
     */
    function recordRequestInsertAccountEvent( $uid )
    {
        $obj =& $this->create();
        $obj->setVar( 'event_type_id', ETID_REQUEST_INSERT_ACCOUNT, true ); // not gpc
        $obj->setVar( 'uid', $uid, true ); // not gpc
        return $this->insert( $obj );
    }

    /**
     * record certify account event (ETID_CERTIFY_ACCOUNT: 16)
     *
     * @access public
     * @param int uid certification target user id
     */
    function recordCertifyAccountEvent( $uid )
    {
        $obj =& $this->create();
        $obj->setVar( 'event_type_id', ETID_CERTIFY_ACCOUNT, true ); // not gpc
        $obj->setVar( 'exec_uid', $this->getExecUid(), true ); // not gpc
        $obj->setVar( 'uid', $uid, true ); // not gpc
        return $this->insert( $obj );
    }

    /**
     * record insert group member event (ETID_INSERT_GROUP_MEMBER: 20)
     *
     * @access public
     * @param int uid joining user id
     * @param int gid joining group id
     */
    function recordInsertGroupMemberEvent( $uid, $gid )
    {
        $obj =& $this->create();
        $obj->setVar( 'event_type_id', ETID_INSERT_GROUP_MEMBER, true ); // not gpc
        $obj->setVar( 'exec_uid', $this->getExecUid(), true ); // not gpc
        $obj->setVar( 'uid', $uid, true ); // not gpc
        $obj->setVar( 'gid', $gid, true ); // not gpc
        return $this->insert( $obj );
    }

    /**
     * record view top page event (ETID_VIEW_TOP_PAGE: 22)
     *
     * @access public
     */
    function recordViewTopPageEvent()
    {
        // get start page script name
        global $xoopsConfig, $xoopsRequestUri;
        $startpage_url = XOOPS_URL.'/index.php';
        if ( isset( $xoopsConfig['startpage'] ) && $xoopsConfig['startpage'] != '' && $xoopsConfig['startpage'] != '--' ) {
          $startpage_url = XOOPS_URL.'/modules/'.$xoopsConfig['startpage'].'/index.php';
        }
        $startpage_script = '';
        if ( preg_match( '/^(\S+):\/\/([^\/]+)((\/[^\/]+)*\/index.php)$/', $startpage_url, $matches ) ) {
	  $startpage_script = $matches[3];
	}
        // get current script name
	$current_script = ( isset( $_SERVER['SCRIPT_NAME'] ) ) ? $_SERVER['SCRIPT_NAME'] : '';
        // compare start page script name with current script name
        if ( $startpage_script != $current_script ) {
          // current url is not top page
          return false;
        }

        // record event
        $obj =& $this->create();
        $obj->setVar( 'event_type_id', ETID_VIEW_TOP_PAGE, true ); // not gpc
        $obj->setVar( 'exec_uid', $this->getExecUid(), true ); // not gpc
        return $this->insert( $obj, true ); // force insertion
    }
}

?>
