<?php
// $Revision: 1.1.2.4 $
//  ------------------------------------------------------------------------ //
//  XooNIps - Neuroinformatics Base Platform System                          //
//  Copyright (C) 2005-2007 RIKEN, Japan All rights reserved.                //
//  http://xoonips.sourceforge.jp/                                           //
//  ------------------------------------------------------------------------ //
//  This program is free software; you can redistribute it and/or modify     //
//  it under the terms of the GNU General Public License as published by     //
//  the Free Software Foundation; either version 2 of the License, or        //
//  (at your option) any later version.                                      //
//                                                                           //
//  You may not change or alter any portion of this comment or credits       //
//  of supporting developers from this source code or any supporting         //
//  source code which is considered copyrighted (c) material of the          //
//  original comment or credit authors.                                      //
//                                                                           //
//  This program is distributed in the hope that it will be useful,          //
//  but WITHOUT ANY WARRANTY; without even the implied warranty of           //
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            //
//  GNU General Public License for more details.                             //
//                                                                           //
//  You should have received a copy of the GNU General Public License        //
//  along with this program; if not, write to the Free Software              //
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA //
//  ------------------------------------------------------------------------ //

include_once XOOPS_ROOT_PATH."/modules/xoonips/class/base/tableobject.class.php";

/**
 * @brief handler class of item type
 *
 * make subclass for each itemtype
 *
 * @li getVar('item_type_id') : item type id
 * @li getVar('name') : item type name
 * @li getVar('display_name') : display name of item type
 * @li getVar('viewphp') : file path of view.php
 *
 * detail of fields
 * @li name : field name string
 * @li type : data type string(int|string|date|fileid)
 * @li required : true=required, false=optional
 * @li multiple : true=multiple, false=not multiple
 */
class XooNIpsItemType extends XooNIpsTableObject
{
    var $fields;
    var $description = null;
    var $mainFileName = null;
    var $previewFileName = null;
    
    var $iteminfo = null;
    
    function XooNIpsItemType( $module = null ) 
    {
        parent::XooNIpsTableObject();
        if( isset( $module ) && is_null( $this -> iteminfo ) ){
            include XOOPS_ROOT_PATH . '/modules/' . $module . '/iteminfo.php';
            $this -> iteminfo =& $iteminfo;
            $this->description = $iteminfo['description'];
            $this->mainFileName = isset( $iteminfo['files']['main'] ) ? $iteminfo['files']['main'] : null;
            $this->previewFileName = isset( $iteminfo['files']['preview'] ) ? $iteminfo['files']['preview'] : null;
        }
        $this->fields = array(
            array( 'name' => 'item_id'           , 'type' => 'int'   , 'required' => true , 'multiple' => false ), //
            array( 'name' => 'item_type_id'      , 'type' => 'int'   , 'required' => true , 'multiple' => false ), //
            array( 'name' => 'uid'               , 'type' => 'int'   , 'required' => true , 'multiple' => false ), //
            array( 'name' => 'title'             , 'type' => 'string', 'required' => true , 'multiple' => true  ), //
            array( 'name' => 'keywords'          , 'type' => 'string', 'required' => false, 'multiple' => true  ), //
            array( 'name' => 'description'       , 'type' => 'string', 'required' => false, 'multiple' => false ), //
            array( 'name' => 'doi'               , 'type' => 'string', 'required' => false, 'multiple' => false ), //
            array( 'name' => 'last_update_date'  , 'type' => 'int'   , 'required' => false, 'multiple' => false ), //
            array( 'name' => 'creation_date'     , 'type' => 'int'   , 'required' => false, 'multiple' => false ), //
            array( 'name' => 'publication_year'  , 'type' => 'int'   , 'required' => false, 'multiple' => false ), //
            array( 'name' => 'publication_month' , 'type' => 'int'   , 'required' => false, 'multiple' => false ), //
            array( 'name' => 'publication_mday'  , 'type' => 'int'   , 'required' => false, 'multiple' => false ), //
            array( 'name' => 'lang'              , 'type' => 'string', 'required' => false, 'multiple' => false ), //
            array( 'name' => 'index_id'          , 'type' => 'int'   , 'required' => true , 'multiple' => true  ), //
            array( 'name' => 'item_id'           , 'type' => 'int'   , 'required' => false, 'multiple' => true  ), //
        );
        if( !is_null($this -> iteminfo['ormfield']['detail']) ){
            $this->fields = array_merge( $this->fields, $this -> iteminfo['ormfield']['detail'] );
        }

        
        $this->initVar('item_type_id'    , XOBJ_DTYPE_INT   , null, true ,   10);
        $this->initVar('name'            , XOBJ_DTYPE_TXTBOX, null, true ,   30);
        $this->initVar('mid'             , XOBJ_DTYPE_INT, null, true);
        $this->initVar('display_name'    , XOBJ_DTYPE_TXTBOX, null, true ,   30);
        $this->initVar('viewphp'         , XOBJ_DTYPE_TXTBOX, null, true ,  255);
    }
    
    /**
     * get field information by name
     * @param $ormName field name of orm
     * @param $fieldName field name
     * @return field information if found. false if not found.
     */
    function getFieldByName( $ormName, $fieldName ){
        // return false if $ormName is not found in iteminfo
        if( !in_array( $ormName, array_keys( $this -> iteminfo['ormfield'] ) ) ) return false;
        // find fieldName in iteminfo['ormfield']
        foreach ( $this -> iteminfo['ormfield'][$ormName] as $field ){
            if ( $field['name'] == $fieldName ){
                return $field;
            }
        }
        return false;
    }
    /**
     * @brief get description
     */
    function getDescription(){
        return $this->description;
    }
    
    /**
     * @brief get field name of mainfile
     */
    function getMainFileName(){
        return $this->mainFileName;
    }
    
    /**
     * @brief get field name of previewfile
     */
    function getPreviewFileName()
    {
        return $this->previewFileName;
    }
    
    /**
     * get iteminfo array
     */
    function getIteminfo(){ return $this -> iteminfo; }
    
    /**
     * return names of all file type used in the item type
     * @return array array of file type name string
     */
    function getFileTypeNames(){
        $ar = array();
        if( isset( $this->iteminfo['files']['main'] ) ) $ar[] = $this->iteminfo['files']['main'];
        if( isset( $this->iteminfo['files']['preview'] ) ) $ar[] = $this->iteminfo['files']['preview'];
        return array_merge( $ar, isset( $this->iteminfo['files']['others'] ) ? $this->iteminfo['files']['others'] : array() );
    }
    
    /**
     * return field has multiple value or not.
     * @param string field name of orm
     * @return boolean true if the field can have multiple value
     */
    function getMultiple( $fieldname ){
        foreach( $this->iteminfo['orm'] as $i ){
            if( $i['field'] == $fieldname ) return isset( $i['multiple'] ) ? $i['multiple'] : false;
        }
        return false;
    }

    /**
     * return that field is required or not.
     * @param string field name of orm
     * @return boolean true if the field is required
     */
    function getRequired( $fieldname ){
        foreach( $this->iteminfo['orm'] as $i ){
            if( $i['field'] == $fieldname ) return isset( $i['required'] ) ? $i['required'] : false;
        }
        return false;
    }
}

/**
 * @brief data object of item type
 *
 * make subclass for each itemtype
 *
 */
class XooNIpsItemTypeHandler extends XooNIpsTableObjectHandler
{
    function XooNIpsItemTypeHandler( &$db ){
        parent::XooNIpsTableObjectHandler( $db );
        $this->__initHandler( 'XooNIpsItemType', 'xoonips_item_type', 'item_type_id', false );
    }
}
?>
