<?php
// $Revision: 1.1.2.8 $
//  ------------------------------------------------------------------------ //
//  XooNIps - Neuroinformatics Base Platform System                          //
//  Copyright (C) 2005-2007 RIKEN, Japan All rights reserved.                //
//  http://xoonips.sourceforge.jp/                                           //
//  ------------------------------------------------------------------------ //
//  This program is free software; you can redistribute it and/or modify     //
//  it under the terms of the GNU General Public License as published by     //
//  the Free Software Foundation; either version 2 of the License, or        //
//  (at your option) any later version.                                      //
//                                                                           //
//  You may not change or alter any portion of this comment or credits       //
//  of supporting developers from this source code or any supporting         //
//  source code which is considered copyrighted (c) material of the          //
//  original comment or credit authors.                                      //
//                                                                           //
//  This program is distributed in the hope that it will be useful,          //
//  but WITHOUT ANY WARRANTY; without even the implied warranty of           //
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            //
//  GNU General Public License for more details.                             //
//                                                                           //
//  You should have received a copy of the GNU General Public License        //
//  along with this program; if not, write to the Free Software              //
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA //
//  ------------------------------------------------------------------------ //

include_once XOOPS_ROOT_PATH . '/modules/xoonips/class/base/relatedobject.class.php';
include_once XOOPS_ROOT_PATH . '/modules/xoonips/condefs.php';
//- - - - - - - - - - - - - - - - - - - - - - - - - - - -
//
// Handlers
//
//- - - - - - - - - - - - - - - - - - - - - - - - - - - -


/**
 *
 * @brief class of item data object handler.
 *
 *
 * @li base class of items depends on xoonips_item_basic table.
 *
 */
class XooNIpsItemCompoHandler extends XooNIpsRelatedObjectHandler
{
    var $db = null;
    function XooNIpsItemCompoHandler(&$db) 
    {
        $this->db = &$db;
        parent::XooNIpsRelatedObjectHandler($db);
        parent::__init_handler('basic', xoonips_getormhandler('xoonips', 'item_basic') , 'item_id');
        $this->addHandler('titles', xoonips_getormhandler('xoonips', 'title') , 'item_id', true);
        $this->addHandler('keywords', xoonips_getormhandler('xoonips', 'keyword') , 'item_id', true);
        $this->addHandler('indexes', xoonips_getormhandler('xoonips', 'index_item_link') , 'item_id', true);
        $this->addHandler('changelogs', xoonips_getormhandler('xoonips', 'changelog') , 'item_id', true);
        $this->addHandler('related_tos', xoonips_getormhandler('xoonips', 'related_to') , 'parent_id', true);
    }
    function &create() 
    {
        $item = &new XooNIpsItemCompo();
        return $item;
    }

    /**
     * gets a value object
     *
     * @param string $ext_id extended item_id
     * @retval XooNIpsItemCompo
     * @retval false
     */
    function &getByExtId($id) 
    {
        static $falseVar = false;
        //
        $handler = &xoonips_getormhandler('xoonips', 'item_basic');
        $objs = &$handler->getObjects(new Criteria('doi', addslashes($id)));
        if (!$objs || 1 != count($objs)) return $falseVar;
        //
        $obj = &$this->get($objs[0]->get('item_id'));
        if ($obj) return $obj;
        //
        return $falseVar;
    }

    /**
     *
     * return true if permitted to this item
     *
     * @param id id of item
     * @param uid uid who access to this item
     * @param operation read|write|delete
     * @return true if permitted
     *
     */
    function getPerm($id, $uid, $operation) 
    {
        $handler = new XooNIpsItemInfoCompoHandler($this->db);
        return $handler->getPerm($id,$uid,$operation);
    }
}

/**
 *
 * @brief class of item data object handler according to iteminfo
 *
 *
 * @li base class of items depends on xoonips_item_basic table.
 *
 */
class XooNIpsItemInfoCompoHandler extends XooNIpsRelatedObjectHandler
{
    var $iteminfo = null;
    var $db = null;
    function XooNIpsItemInfoCompoHandler(&$db, $module = null) 
    {
        parent::XooNIpsRelatedObjectHandler($db);
        $this->db = &$db;
        if (isset($module) && is_null($this->iteminfo)) {
            include XOOPS_ROOT_PATH . '/modules/' . $module . '/iteminfo.php';
            $this->iteminfo = &$iteminfo;
            //
            // add orm handler according to $iteminfo['orm']
            foreach($this->iteminfo['orm'] as $orminfo) {
                if ($orminfo['field'] == $this->iteminfo['ormcompo']['primary_orm']) { //orm of primary table
                    parent::__init_handler($orminfo['field'], xoonips_getormhandler($orminfo['module'], $orminfo['name']) , $orminfo['foreign_key']);
                } else {
                    $this->addHandler($orminfo['field'], xoonips_getormhandler($orminfo['module'], $orminfo['name']) , $orminfo['foreign_key'], isset($orminfo['multiple']) ? $orminfo['multiple'] : false, isset($orminfo['criteria']) ? $orminfo['criteria'] : null);
                }
            }
        }
    }

    /**
     * gets a value object
     *
     * @param string $ext_id extended item_id
     * @retval XooNIpsItemInfoCompo
     * @retval false
     */
    function &getByExtId($id) 
    {
        static $falseVar = false;
        //
        $handler = &xoonips_getormhandler('xoonips', 'item_basic');
        $objs = &$handler->getObjects(new Criteria('doi', addslashes($id)));
        if (!$objs || 1 != count($objs)) return $falseVar;
        //
        $obj = &$this->get($objs[0]->get('item_id'));
        if ($obj) return $obj;
        //
        return $falseVar;
    }

    /**
     *
     * return true if permitted to this item
     *
     * @param id id of item
     * @param uid uid who access to this item
     * @param operation read|write|delete
     * @return true if permitted
     *
     */
    function getPerm($id, $uid, $operation) 
    {
        if (!in_array($operation, array(
            'read',
            'write',
            'delete'
        ))) {
            return false; // bad operation
            
        }
        $item_basic_handler = &xoonips_getormhandler('xoonips', 'item_basic');
        $item_basic = &$item_basic_handler->get($id);
        if (!$item_basic || $item_basic->get('item_type_id') == ITID_INDEX ) {
            return false; // no such item
        }
        $member_handler = &xoonips_gethandler('member');
        $is_moderator = $member_handler->isModerator($uid);
        if ($is_moderator && ($operation == 'write' || $operation == 'read')) {
            $config_handler = &xoonips_getormhandler('xoonips', 'config');
            $config = &$config_handler->get('moderator_modify_any_items'); // todo: define constant string
            if ($config && $config->get('value') == 'on') {
                return true; // moderator && moderator can modfy any items
                
            }
        }
        if ($item_basic->get('uid') == $uid) {
            return true; // my own item
            
        }
        if ($operation == 'read') {
            // item_basic <- index_item_link <- index <- groups_users_link
            $join1 = new XoopsJoinCriteria($this->db->prefix('xoonips_index_item_link') , 'item_id', 'item_id', 'left', 'txil');
            $join2 = new XoopsJoinCriteria($this->db->prefix('xoonips_index') , 'index_id', 'index_id', 'left', 'tx');
            $join3 = new XoopsJoinCriteria($this->db->prefix('xoonips_groups_users_link') , 'gid', 'gid', 'left', 'tgul');
            $join1->cascade($join2);
            $join2->cascade($join3);
            $criteria = new CriteriaCompo();
            if ($is_moderator) {
                $criteria->add(new Criteria('open_level', OL_PUBLIC, '=', 'tx') , 'and'); // public index && moderator
                
            } else {
                $criteria->add(new Criteria('open_level', OL_PUBLIC, '=', 'tx') , 'and');
                $criteria->add(new Criteria('certify_state', CERTIFIED, '=', 'txil') , 'and'); // public index && certified
                
            }
            $criteria->add(new Criteria('open_level', OL_GROUP_ONLY, '=', 'tx') , 'or'); // group index && group admin
            $criteria->add(new Criteria('uid', $uid, '=', 'tgul') , 'and');
            $criteria->add(new Criteria('is_admin', 1, '=', 'tgul') , 'and');
            $criteria->add(new Criteria('open_level', OL_GROUP_ONLY, '=', 'tx') , 'or'); // group index && group member && certified
            $criteria->add(new Criteria('uid', $uid, '=', 'tgul') , 'and');
            $criteria->add(new Criteria('certify_state', CERTIFIED, '=', 'txil') , 'and');
            $criteria->add(new Criteria('open_level', OL_PRIVATE, '=', 'tx') , 'or'); // private index && mine
            $criteria->add(new Criteria('uid', $uid, '=', 'tx') , 'and');
            $criteria = new CriteriaCompo($criteria);
            $criteria->add(new Criteria('item_id', $id, '=', 'txil' ));
            $item_basics = $item_basic_handler->getObjects($criteria, false, '', null, $join1);
            if ($item_basics && count($item_basics)) return true;
        }
        return false;
    }

    /**
     * @brief search item
     *
     * @param query query ( string or CriteriaElement )
     * @param limit the maximum number of rows to return(0 = no limit)
     * @param offset the offset of the first row to return(0 = from beginning)
     * @param uid user ID
     * @return array of item id
     */
    function search($query, $limit, $offset, $uid) 
    {
        if ( !$this->iteminfo ){
            return array();
        }
        
        $modulename = $this->iteminfo['ormcompo']['module'];
        $dummy = false;
        $search_cache_id = false;
        // save xoopsUser
        if (isset($GLOBALS['xoopsUser'])) {
            $old_xoopsUser = $GLOBALS['xoopsUser'];
        } else {
            $old_xoopsUser = null;
        }
        // prepare for xnpSearchExec
        $member_handler = xoops_gethandler('member');
        $GLOBALS['xoopsUser'] = $member_handler->getUser($uid);
        // search
        $item_ids = array();
        if (xnpSearchExec('quicksearch', $query, $modulename, false, $dummy, $dummy, $dummy, $search_cache_id, false, 'item_metadata')) {
            $search_cache_item_handler = &xoonips_getormhandler('xoonips', 'search_cache_item');
            $criteria = new Criteria('search_cache_id', $search_cache_id);
            $criteria->setSort('item_id');
            $criteria->setStart($offset);
            if ($limit) {
                $criteria->setLimit($limit);
            }
            $search_cache_items = $search_cache_item_handler->getObjects($criteria);
            foreach($search_cache_items as $search_cache_item) {
                $item_ids[] = $search_cache_item->get('item_id');
            }
        }
        // restore xoopsUser
        $GLOBALS['xoopsUser'] = $old_xoopsUser;
        return $item_ids;
    }

    /**
     * get iteminfo array
     */
    function getIteminfo() 
    {
        return $this->iteminfo;
    }
}
//- - - - - - - - - - - - - - - - - - - - - - - - - - - -
//
// Data object
//
//- - - - - - - - - - - - - - - - - - - - - - - - - - - -


/**
 *
 * @brief data object of xoonips item(basic fields)
 *
 * @li getVar('basic') : {@link XooNIpsItemCompoBasic}
 * @li getVar('titles') : array of {@link XooNIpsTitle}
 * @li getVar('keywords') : array of {@link XooNIpsKeyword}
 * @li getVar('related_tos') : array of {@link XooNIpsRelatedTo}
 * @li getVar('changelogs') : array of {@link XooNIpsChangelog}
 * @li getVar('indexes') : array of {@link XooNIpsIndexItemLink}
 *
 */
class XooNIpsItemCompo extends XooNIpsRelatedObject
{
    function XooNIpsItemCompo() 
    {
        // basic
        $basic_handler = &xoonips_getormhandler('xoonips', 'item_basic');
        $this->initVar('basic', $basic_handler->create() , true);
        $this->initVar('titles', $titles = array() , true);
        $this->initVar('keywords', $keywrods = array());
        $this->initVar('related_tos', $related_tos = array());
        $this->initVar('changelogs', $changelogs = array());
        $this->initVar('indexes', $indexes = array() , true);
    }
}

/**
 *
 * @brief data object of xoonips item according to iteminfo
 *
 */
class XooNIpsItemInfoCompo extends XooNIpsRelatedObject
{
    var $iteminfo = null;
    function XooNIpsItemInfoCompo($module = null) 
    {
        if (isset($module) && is_null($this->iteminfo)) {
            include XOOPS_ROOT_PATH . '/modules/' . $module . '/iteminfo.php';
            $this->iteminfo = &$iteminfo;
            // add orm object according to $this -> iteminfo['orm']
            foreach($this->iteminfo['orm'] as $orminfo) {
                $handler = &xoonips_getormhandler($orminfo['module'], $orminfo['name']);
                if (isset($orminfo['multiple']) ? $orminfo['multiple'] : false) {
                    $ary = array();
                    $this->initVar($orminfo['field'], $ary, isset($orminfo['required']) ? $orminfo['required'] : false);
                    unset($ary);
                } else $this->initVar($orminfo['field'], $handler->create() , isset($orminfo['required']) ? $orminfo['required'] : false);
            }
        }
    }

    /**
     * get iteminfo array
     */
    function getIteminfo() 
    {
        return $this->iteminfo;
    }
}
?>
