<?php
//  $Revision: 1.1.2.7 $
//  ------------------------------------------------------------------------ //
//  XooNIps - Neuroinformatics Base Platform System                          //
//  Copyright (C) 2005-2007 RIKEN, Japan All rights reserved.                //
//  http://xoonips.sourceforge.jp/                                           //
//  ------------------------------------------------------------------------ //
//  This program is free software; you can redistribute it and/or modify     //
//  it under the terms of the GNU General Public License as published by     //
//  the Free Software Foundation; either version 2 of the License, or        //
//  (at your option) any later version.                                      //
//                                                                           //
//  You may not change or alter any portion of this comment or credits       //
//  of supporting developers from this source code or any supporting         //
//  source code which is considered copyrighted (c) material of the          //
//  original comment or credit authors.                                      //
//                                                                           //
//  This program is distributed in the hope that it will be useful,          //
//  but WITHOUT ANY WARRANTY; without even the implied warranty of           //
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            //
//  GNU General Public License for more details.                             //
//                                                                           //
//  You should have received a copy of the GNU General Public License        //
//  along with this program; if not, write to the Free Software              //
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA //
//  ------------------------------------------------------------------------ //

if ( ! defined( 'XOOPS_ROOT_PATH' ) ) exit();

require_once XOOPS_ROOT_PATH.'/modules/xoonips/condefs.php';
require_once XOOPS_ROOT_PATH.'/modules/xoonips/class/base/tableobject.class.php';
require_once XOOPS_ROOT_PATH.'/modules/xoonips/include/functions.php';


/**
 * create xoops group
 *
 * @param string $name group name
 * @param string $description group description
 * @param string $errmes error message, 
 *               set message to this variable if creation failed
 * @return bool false if failure
 */
function xoonips_create_xoops_group( $name, $description ) {
  $member_handler =& xoops_gethandler( 'member' );
  $group =& $member_handler->createGroup();
  $group->setVar( 'name', $name );
  $group->setVar( 'description', $description );
  $ret = $member_handler->insertGroup( $group );
  if ( $ret == false ) {
    return false;
  }
  $gid = $group->getVar( 'groupid', 'n' );
  return $gid;
}


/**
 * get xoonips module id
 *
 * @return int module id, if an error occured then return false
 */
function xoonips_get_module_id() {
  $module_handler =& xoops_gethandler( 'module' );
  $module =& $module_handler->getByDirname( 'xoonips' );
  if ( ! is_object( $module ) ) {
    return false;
  }
  $mid = $module->getVar( 'mid' );
  return $mid;
}


/**
 * set block right
 *
 * @param int $bid block id
 * @param int $gid group id
 * @param bool $right has read right?
 * @return bool false if failure
 */
function xoonips_set_block_right( $bid, $gid, $right ) {
  $criteria = new CriteriaCompo( new Criteria( 'gperm_name', 'block_read' ) );
  $criteria->add( new Criteria( 'gperm_groupid', $gid ) );
  $criteria->add( new Criteria( 'gperm_itemid', $bid ) );
  $criteria->add( new Criteria( 'gperm_modid', 1 ) );
  $gperm_handler =& xoops_gethandler( 'groupperm' );
  $gperm_objs =& $gperm_handler->getObjects( $criteria );
  if ( count( $gperm_objs ) > 0 ) {
    // already exists
    $gperm_obj = $gperm_objs[0];
    if ( ! $right ) {
      $gperm_handler->delete( $gperm_obj );
    }
  } else {
    // not found
    if ( $right ) {
      $gperm_handler->addRight( 'block_read' , $bid, $gid );
    }
  }
  return true;
}


/**
 * set block positions
 *
 * @param int $bid block id
 * @param bool $visible visible flag
 * @param int $side
 *        0: sideblock - left
 *        1: sideblock - right
 *        2: sideblock - left and right
 *        3: centerblock - left
 *        4: centerblock - right
 *        5: centerblock - center
 *        6: centerblock - left, right, center
 * @param int $weight weight
 * @return bool false if failure
 */
function xoonips_set_block_positions( $bid, $visible, $side, $weight ) {
  $block = new XoopsBlock();
  $block->load( $bid );
  $block->setVar( 'visible', $visible ? 1 : 0 );
  $block->setVar( 'side', $side );
  $block->setVar( 'weight', $weight );
  return $block->store();
}


/**
 * set block show page
 *
 * @param int $bid block id
 * @param int $mid
 *     -1 : top page
 *      0 : all pages
 *    >=1 : module id
 * @param bool $is_show 
 */
function xoonips_set_block_show_page( $bid, $mid, $is_show ) {
  global $xoopsDB;
  $table = $xoopsDB->prefix( 'block_module_link' );
  // check current status
  $sql = sprintf( 'SELECT block_id, module_id FROM %s WHERE block_id = %u AND module_id = %d', $table, $bid, $mid );
  if ( ! $result = $xoopsDB->query( $sql ) ) {
    return false;
  }
  $count = $xoopsDB->getRowsNum( $result );
  $xoopsDB->freeRecordSet( $result );
  if ( $count == 0 ) {
    // not exists
    if ( $is_show ) {
      $sql = sprintf( 'INSERT INTO %s (block_id, module_id) VALUES ( %u, %d )', $table, $bid, $mid );
      if ( ! $result = $xoopsDB->query( $sql ) ) {
        return false;
      }
    }
  } else {
    // already exists
    if ( ! $is_show ) {
      $sql = sprintf( 'DELETE FROM %s WHERE block_id = %u AND module_id = %d', $table, $bid, $mid );
      if ( ! $result = $xoopsDB->query( $sql ) ) {
        return false;
      }
    }
  }
  return true;
}


/**
 * set start module
 *
 * @param string $dirname module directory name,  '--' means no module.
 * @return bool false if failure
 */
function xoonips_set_startup_page( $dirname ) {
  $config_handler =& xoops_gethandler('config');
  $criteria = new CriteriaCompo( new Criteria( 'conf_modid', 0 ) );
  $criteria->add( new Criteria('conf_catid', XOOPS_CONF ) );
  $criteria->add( new Criteria('conf_name', 'startpage') );
  $configs =& $config_handler->getConfigs($criteria);
  if ( count( $configs ) != 1 ){
    return false;
  }
  $config = $configs[0];
  $config->setConfValueForInput( $dirname );
  return $config_handler->insertConfig($config);
}


/**
 * set module read right
 *
 * @param int $mid module id
 * @param int $gid group id
 * @param int $right has read right ?
 * @return bool false if failure
 */
function xoonips_set_module_right( $mid, $gid, $right ) {
  $criteria = new CriteriaCompo( new Criteria( 'gperm_name', 'module_read' ) );
  $criteria->add( new Criteria( 'gperm_groupid', $gid ) );
  $criteria->add( new Criteria( 'gperm_itemid', $mid ) );
  $criteria->add( new Criteria( 'gperm_modid', 1 ) );
  $gperm_handler =& xoops_gethandler( 'groupperm' );
  $gperm_objs =& $gperm_handler->getObjects( $criteria );
  if ( count( $gperm_objs ) > 0 ) {
    // already exists
    $gperm_obj = $gperm_objs[0];
    if ( ! $right ) {
      $gperm_handler->delete( $gperm_obj );
    }
  } else {
    // not found
    if ( $right ) {
      $gperm_handler->addRight( 'module_read' , $mid, $gid );
    }
  }
  return true;
}

/**
 * get xoonips config
 *
 * @param string $key config key name
 * @return mixed value, if config not found then null
 */
function xoonips_get_config( $key ) {
  $xconfig_handler =& xoonips_getormhandler( 'xoonips', 'config' );
  $obj =& $xconfig_handler->get( $key );
  if ( $obj === false ) {
    return null;
  }
  $var = $obj->getVar( 'value', 'n' );
  return $var;
}


/**
 * set xoonips config
 *
 * @param string $key config key name
 * @param string $value config value
 * @return bool false if failure
 */
function xoonips_set_config( $key, $value ) {
  $xconfig_handler =& xoonips_getormhandler( 'xoonips', 'config' );
  $obj =& $xconfig_handler->get( $key );
  if ( $obj === false ) {
    // insert new entry
    $obj =& $xconfig_handler->create();
    $obj->setVar( 'name', $key, true ); // not gpc
    $obj->setVar( 'value', $value, true ); // not gpc
    if ( ! $xconfig_handler->insert( $obj ) ) {
      return false;
    }
  } else {
    // update entry
    $obj->setVar( 'value', $value, true ); // not gpc
    if ( ! $xconfig_handler->insert( $obj ) ) {
      return false;
    }
  }
  return true;
}


/**
 * update module id for binder item type
 *
 * @param int $mid module id
 * @return bool false if failure
 */
function xoonips_update_binder_mid( $mid ) {
  $it_handler =& xoonips_getormhandler( 'xoonips', 'item_type' );
  $criteria = new Criteria( 'name', 'xoonips_binder' );
  $it_objs =& $it_handler->getObjects( $criteria );
  if ( count( $it_objs ) != 1 ) {
    return false;
  }
  $it_obj =& $it_objs[0];
  $it_obj->setVar( 'mid', $mid );
  return $it_handler->insert( $it_obj );
}


/**
 * insert initial configuration values
 */
function xoonips_insert_initial_values() {
  $config_values = array(
    // --------------------------------------------
    // System Configurations
    // --------------------------------------------
    //  - Basic
    'moderator_gid' => '1',
    'upload_dir' => '/var/tmp',
    'magic_file_path' => '/usr/share/misc/magic',
    //  - Inline frame
    'tree_frame_width' => '100%',
    'tree_frame_height' => '400',
    //  - Printer
    'printer_friendly_header' => '',
    //  - RSS
    'rss_file_path' => '',
    'rss_item_max' =>'10',
    //  - OAI-PMH
    'repository_name' => '',
    'repository_nijc_code' => '',
    'repository_deletion_track' => '30',
    'repository_institution' => 'meta_author',
    'repository_publisher' => 'meta_author',
    //  - Proxy
    'proxy_host' => '',
    'proxy_port' => '80',
    'proxy_user' => '',
    'proxy_pass' => '',
    // --------------------------------------------
    // Site Policies
    // --------------------------------------------
    //  - User Informations
    'certify_user' => 'on',
    'account_realname_optional' => 'on',
    'account_company_name_optional' => 'on',
    'account_division_optional' => 'on',
    'account_country_optional' => 'on',
    'account_address_optional' => 'on',
    'account_zipcode_optional' => 'on',
    'account_tel_optional' => 'on',
    'account_fax_optional' => 'on',
    'private_item_number_limit' => '100',
    'private_index_number_limit' => '50',
    'private_item_storage_limit' => '50000000',
    //  - Group Informations
    'group_item_number_limit' => '300',
    'group_index_number_limit' => '200',
    'group_item_storage_limit' => '100000000',
    //  - Publications
    'certify_item' => 'on',
    'public_item_target_user' => 'all',
    'download_file_compression' => 'on',
    'item_show_optional' => 'off',
    //  - Import / Export
    'export_attachment' => 'off',
    'export_enabled' => 'off',
    'private_import_enabled' => 'off',
    //  - Moderator Privileges
    'moderator_modify_any_items' => 'off',
    //  - Access Rankings
    'ranking_num_rows' => '5',
    'ranking_order' => '0,1,2,3,4',
    'ranking_visible' => '1,1,1,1,1',
    'ranking_new_num_rows' => '5',
    'ranking_new_order' => '0,1',
    'ranking_new_visible' => '1,1',
    'ranking_days' => '14',
    'ranking_days_enabled' => '',
    'ranking_lock_timeout' => '0',
    'ranking_last_update' => '0',
    'ranking_sum_start' => '0',
    'ranking_sum_last_update' => '0',
  );
  foreach ( $config_values as $key => $val ) {
    xoonips_set_config( $key, $val );
  }
}


/**
 * xoonips install function
 *
 * @param object $xoopsMod module instance
 * @return bool false if failure
 */
function xoops_module_install_xoonips( $xoopsMod ) {
  // insert initial config values
  xoonips_insert_initial_values();

  // get xoonips module id
  $xoonips_mid = xoonips_get_module_id();
  if ( $xoonips_mid === false ) {
    return false;
  }

  // create moderator group
  $groupid_moderator = xoonips_create_xoops_group( 'moderator', 'platform moderator' );
  if ( $groupid_moderator === false ) {
    return false;
  }

  // join myself to moderator group 
  $myuid = $GLOBALS['xoopsUser']->getVar('uid');
  $member_handler =& xoops_gethandler('member');
  $member_handler->addUserToGroup($groupid_moderator, $myuid);
  // update group cache and session
  $mygroups = $member_handler->getGroupsByUser( $myuid );
  $GLOBALS['xoopsUser']->setGroups( $mygroups );
  if ( isset( $_SESSION['xoopsUserGroups'] ) ) {
    $_SESSION['xoopsUserGroups'] = $mygroups;
  }

  // define groups
  $groupid_admin = 1;
  $groupid_user = 2;
  $groupid_guest = 3;
  $groupids_all = array_keys( $member_handler->getGroupList() );
  $groupids_other = array_diff( $groupids_all, array( $groupid_admin, $groupid_user, $groupid_guest, $groupid_moderator ) );

  // set module access permission
  foreach ( $groupids_all as $gid ) {
    if ( in_array( $gid, $groupids_other ) ) {
      xoonips_set_module_right( $xoonips_mid, $gid, false );
    } else {
      xoonips_set_module_right( $xoonips_mid, $gid, true );
    }
  }

  // set block read permissions and positions
  // - block permissions
  $block_perms = array(
    // 'func name' => array( 'registered user', 'guest', 'moderator' );
    'b_xoonips_quick_search_show' => array( true, true, false ),
    'b_xoonips_moderator_show' => array( false, false, true ),
    'b_xoonips_tree_show' => array( true, true, false ),
    'b_xoonips_login_show' => array( true, true, false ),
    'b_xoonips_user_show' => array( true, false, false ),
    'b_xoonips_group_show' => array( true, false, false ),
    'b_xoonips_itemtypes_show' => array( true, true, false ),
    'b_xoonips_ranking_new_show' => array( true, true, false ),
    'b_xoonips_ranking_show' => array( true, true, false ),
    'b_xoonips_userlist_show' => array( true, false, false ),
  );
  // - block positions
  $block_pos = array(
    // 'func name' => array( 'visible', 'side', 'weight' 'show all pages' );
    'b_xoonips_quick_search_show' => array( true, 0, 10, true ),
    'b_xoonips_moderator_show' => array( true, 1, 20, true ),
    'b_xoonips_tree_show' => array( true, 0, 20, true ),
    'b_xoonips_login_show' => array( true, 0, 0, true ),
    'b_xoonips_user_show' => array( true, 1, 0, true ),
    'b_xoonips_group_show' => array( true, 1, 10, true ),
    'b_xoonips_itemtypes_show' => array( true, 5, 20, false ),
    'b_xoonips_ranking_new_show' => array( false, 0, 0, false ),
    'b_xoonips_ranking_show' => array( false, 0, 0, false ),
    'b_xoonips_userlist_show' => array( false, 0, 0, false ),
  );
  // - logic
  $block_objs =& XoopsBlock::getByModule( $xoonips_mid );
  foreach ( $block_objs as $block_obj ) {
    $func = $block_obj->getVar( 'show_func', 'n' );
    $bid = $block_obj->getVar( 'bid', 'n' );
    // set permissions
    if ( isset( $block_perms[$func] ) ) {
      list( $perm_user, $perm_guest, $perm_moderator ) = $block_perms[$func];
      xoonips_set_block_right( $bid, $groupid_user, $perm_user );
      xoonips_set_block_right( $bid, $groupid_guest, $perm_guest );
      xoonips_set_block_right( $bid, $groupid_moderator, $perm_moderator );
    }
    // set positions
    if ( isset( $block_pos[$func] ) ) {
      list( $b_visible, $b_side, $b_weight, $b_allpage ) = $block_pos[$func];
      xoonips_set_block_positions( $bid, $b_visible, $b_side, $b_weight );
      xoonips_set_block_show_page( $bid, 0, $b_allpage );
      if ( $b_allpage ) {
        // unset top page
        xoonips_set_block_show_page( $bid, -1, false );
      }
    }
    // removed block permissions from other groups
    foreach( $groupids_other as $gid ) {
      xoonips_set_block_right( $bid, $gid, false );
    }
  }

  // hide system blocks
  $sys_blocks = array(
    // 'bid' => array( 'visible', 'side', 'weight' 'show all pages' );
    1 => array( false, 0, 0, true ), // b_system_user_show
    2 => array( false, 0, 0, true ), // b_system_login_show
  );
  foreach ( $sys_blocks as $bid => $block_pos ) {
    list( $b_visible, $b_side, $b_weight, $b_allpage ) = $block_pos;
    xoonips_set_block_positions( $bid, $b_visible, $b_side, $b_weight );
  }

  // set start up page to xoonips module
  //if ( ! xoonips_set_startup_page( 'xoonips' ) ) {
  //  return false;
  //}

  // set moderator id to xoonips config
  if ( ! xoonips_set_config( 'moderator_gid', $groupid_moderator ) ) {
    return false;
  }

  // register mid in binder
  if ( ! xoonips_update_binder_mid( $xoonips_mid ) ) {
    return false;
  }

  // register my xoonips user information
  $xmember_handler =& xoonips_gethandler( 'member' );
  if ( ! $xmember_handler->pickupXoopsUser( $myuid, true ) ) {
    return false;
  }

  return true;
} 

?>
